-------------------------------------------------------------------------------------
-- FILE NAME : testbench_template.vhd
-- AUTHOR    : Luis
-- COMPANY   : 
-- UNITS     : Entity       - testbench_template
--             Architecture - Behavioral
-- LANGUAGE  : VHDL
-- DATE      : May 21, 2010
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
-- DESCRIPTION
-- ===========
-- This entity is template for writing test benches
--
--
-------------------------------------------------------------------------------------
 
-------------------------------------------------------------------------------------
-- LIBRARIES
-------------------------------------------------------------------------------------
library ieee;
   use ieee.std_logic_1164.all;
   use ieee.std_logic_unsigned.all;
   use ieee.std_logic_misc.all;

library xil_defaultlib;
-------------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------------
entity axi_fmc110_tb is
end axi_fmc110_tb;

-------------------------------------------------------------------------------------
-- ARCHITECTURE
-------------------------------------------------------------------------------------
architecture Behavioral of axi_fmc110_tb is

-------------------------------------------------------------------------------------
-- CONSTANTS
-------------------------------------------------------------------------------------
constant CLK_200_MHZ : time := 5 ns;
constant CLK_100_MHZ : time := 10 ns;
constant CLK_300_MHZ : time := 3.3333 ns;
constant CLK_25_MHZ  : time := 40 ns;
constant CLK_167_MHZ : time := 6 ns;

constant DATA_WIDTH  : natural := 8;
constant ADDR_WIDTH  : natural := 8;


--COMPONENT axi_traffic_gen_0
--  PORT (
--    s_axi_aclk : IN STD_LOGIC;
--    s_axi_aresetn : IN STD_LOGIC;
--    m_axi_lite_ch1_awaddr : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
--    m_axi_lite_ch1_awprot : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
--    m_axi_lite_ch1_awvalid : OUT STD_LOGIC;
--    m_axi_lite_ch1_awready : IN STD_LOGIC;
--    m_axi_lite_ch1_wdata : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
--    m_axi_lite_ch1_wstrb : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
--    m_axi_lite_ch1_wvalid : OUT STD_LOGIC;
--    m_axi_lite_ch1_wready : IN STD_LOGIC;
--    m_axi_lite_ch1_bresp : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
--    m_axi_lite_ch1_bvalid : IN STD_LOGIC;
--    m_axi_lite_ch1_bready : OUT STD_LOGIC;
--    m_axi_lite_ch1_araddr : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
--    m_axi_lite_ch1_arvalid : OUT STD_LOGIC;
--    m_axi_lite_ch1_arready : IN STD_LOGIC;
--    m_axi_lite_ch1_rdata : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
--    m_axi_lite_ch1_rvalid : IN STD_LOGIC;
--    m_axi_lite_ch1_rresp : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
--    m_axi_lite_ch1_rready : OUT STD_LOGIC;
--    done : OUT STD_LOGIC;
--    status : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
--  );
--END COMPONENT;
--ATTRIBUTE SYN_BLACK_BOX : BOOLEAN;
--ATTRIBUTE SYN_BLACK_BOX OF axi_traffic_gen_0 : COMPONENT IS TRUE;
--ATTRIBUTE BLACK_BOX_PAD_PIN : STRING;
--ATTRIBUTE BLACK_BOX_PAD_PIN OF axi_traffic_gen_0 : COMPONENT IS "s_axi_aclk,s_axi_aresetn,m_axi_lite_ch1_awaddr[31:0],m_axi_lite_ch1_awprot[2:0],m_axi_lite_ch1_awvalid,m_axi_lite_ch1_awready,m_axi_lite_ch1_wdata[31:0],m_axi_lite_ch1_wstrb[3:0],m_axi_lite_ch1_wvalid,m_axi_lite_ch1_wready,m_axi_lite_ch1_bresp[1:0],m_axi_lite_ch1_bvalid,m_axi_lite_ch1_bready,m_axi_lite_ch1_araddr[31:0],m_axi_lite_ch1_arvalid,m_axi_lite_ch1_arready,m_axi_lite_ch1_rdata[31:0],m_axi_lite_ch1_rvalid,m_axi_lite_ch1_rresp[1:0],m_axi_lite_ch1_rready,done,status[31:0]";




-----------------------------------------------------------------------------------
-- SIGNALS
-----------------------------------------------------------------------------------
signal sysclk_p         : std_logic := '1';
signal sysclk_n         : std_logic := '0';
signal clk              : std_logic := '1';

signal rst              : std_logic := '1';
signal rstn             : std_logic := '0';

--stimulus
signal sim_count        : std_logic_vector(31 downto 0) := (others=>'0');

signal we               : std_logic;
signal data_rst         : std_logic := '1';

signal dac0_tdata             : std_logic_vector(63 downto 0); 
signal dac0_tkeep             : std_logic_vector(3 downto 0); 
signal dac0_tlast             : std_logic; 
signal dac0_tready            : std_logic;
signal dac0_tstrb             : std_logic;
signal dac0_tuser             : std_logic_vector(31 downto 0);
signal dac0_tvalid            : std_logic;

signal adc0_tdata             : std_logic_vector(63 downto 0); 
signal adc0_tkeep             : std_logic_vector(3 downto 0); 
signal adc0_tlast             : std_logic; 
signal adc0_tready            : std_logic;
signal adc0_tstrb             : std_logic;
signal adc0_tuser             : std_logic_vector(31 downto 0);
signal adc0_tvalid            : std_logic;


signal dac0_enable            : std_logic;

signal cmdclk_out  : std_logic;
signal cmd_in      : std_logic_vector(63 downto 0);
signal cmd_in_val  : std_logic;
signal cmd_out     : std_logic_vector(63 downto 0);
signal cmd_out_val : std_logic;
 
signal axi_cmd_out     : std_logic_vector(63 downto 0);
signal axi_cmd_out_val : std_logic;

--AXI-Lite to 4DSP Stellar Cmds Signals
signal axi_aclk              : std_logic;
signal axi_aresetn           : std_logic;
signal axi_awaddr            : std_logic_vector(32-1 downto 0);
signal axi_awvalid           : std_logic;
signal axi_awready           : std_logic;
signal axi_wdata             : std_logic_vector(31  downto 0);                        
signal axi_wstrb             : std_logic_vector(3  downto 0);                        
signal axi_wvalid            : std_logic;
signal axi_wready            : std_logic;
signal axi_bresp             : std_logic_vector(1 downto 0);
signal axi_bvalid            : std_logic;
signal axi_bready            : std_logic;
signal axi_araddr            : std_logic_vector(31  downto 0);                                 
signal axi_arvalid           : std_logic;
signal axi_arready           : std_logic;
signal axi_rdata             : std_logic_vector(31 downto 0);
signal axi_rresp             : std_logic_vector(1 downto 0);
signal axi_rvalid            : std_logic;
signal axi_rready            : std_logic;

signal fmc_to_cpld      : std_logic_vector(3 downto 0);
signal front_io_fmc     : std_logic_vector(3 downto 0);
signal clk_to_fpga_p    : std_logic;
signal clk_to_fpga_n    : std_logic;
signal ext_trigger_p    : std_logic;
signal ext_trigger_n    : std_logic;
signal sync_from_fpga_p : std_logic;
signal sync_from_fpga_n : std_logic;
signal adc0_clka_p      : std_logic;
signal adc0_clka_n      : std_logic;
signal adc0_da_p        : std_logic_vector(11 downto 0);
signal adc0_da_n        : std_logic_vector(11 downto 0);
signal adc0_ovra_p      : std_logic;
signal adc0_ovra_n      : std_logic;
signal adc0_db_p        : std_logic_vector(11 downto 0);
signal adc0_db_n        : std_logic_vector(11 downto 0);
signal adc0_ovrb_p      : std_logic;
signal adc0_ovrb_n      : std_logic;
signal adc1_clka_p      : std_logic;
signal adc1_clka_n      : std_logic;
signal adc1_da_p        : std_logic_vector(11 downto 0);
signal adc1_da_n        : std_logic_vector(11 downto 0);
signal adc1_ovra_p      : std_logic;
signal adc1_ovra_n      : std_logic;
signal dac_sync_p       : std_logic;
signal dac_sync_n       : std_logic;
signal dac0_dclk_p      : std_logic;
signal dac0_dclk_n      : std_logic;
signal dac0_data_p      : std_logic_vector(15 downto 0);
signal dac0_data_n      : std_logic_vector(15 downto 0);
signal dac1_dclk_p      : std_logic;
signal dac1_dclk_n      : std_logic;
signal dac1_data_p      : std_logic_vector(15 downto 0);
signal dac1_data_n      : std_logic_vector(15 downto 0);
signal pg_m2c           : std_logic;
signal prsnt_m2c_l      : std_logic;

signal i2c_scl          : std_logic;
signal i2c_sda          : std_logic;

-- Traffic Generator Signals
signal done :  std_logic;
signal status : std_logic_vector(31 DOWNTO 0);



signal adc0_count       : std_logic_vector(15 downto 0) := (others=>'0');

--***********************************************************************************
begin
--***********************************************************************************
-- Clock & reset generation
sysclk_p <= not sysclk_p after CLK_100_MHZ/2;
sysclk_n <= not sysclk_p;

clk      <= not clk after CLK_100_MHZ / 2;
rst      <= '0'     after CLK_100_MHZ * 10;
rstn     <= '1'     after CLK_100_MHZ * 10;


axi_aclk    <= clk;
axi_aresetn <= rstn;


--------------------------------------------------------------------------------------
-- AXI_FMC110 IP
---------------------------------------------------------------------------------------
inst_axi_fmc110:
entity xil_defaultlib.axi_fmc110
port map (
   s_axi_aclk       =>  axi_aclk,
   s_axi_aresetn    =>  axi_aresetn,
   -- axi-lite command interface
   s_axi_awaddr     =>  axi_awaddr,
   s_axi_awvalid    =>  axi_awvalid,
   s_axi_awready    =>  axi_awready,
   s_axi_wdata      =>  axi_wdata,
   s_axi_wstrb      =>  axi_wstrb,
   s_axi_wvalid     =>  axi_wvalid,
   s_axi_wready     =>  axi_wready,
   s_axi_bresp      =>  axi_bresp,
   s_axi_bvalid     =>  axi_bvalid,
   s_axi_bready     =>  axi_bready,
   s_axi_araddr     =>  axi_araddr,
   s_axi_arvalid    =>  axi_arvalid,
   s_axi_arready    =>  axi_arready,
   s_axi_rdata      =>  axi_rdata,
   s_axi_rresp      =>  axi_rresp,
   s_axi_rvalid     =>  axi_rvalid,
   s_axi_rready     =>  axi_rready,
   -- axi-stream slave interface for DAC0
   dac0_tdata       => dac0_tdata,
   dac0_tkeep       => dac0_tkeep,     
   dac0_tlast       => dac0_tlast,     
   dac0_tready      => dac0_tready,    
   dac0_tstrb       => dac0_tstrb,     
   dac0_tuser       => dac0_tuser,     
   dac0_tvalid      => dac0_tvalid,  
   -- axi-stream master interface for ADC0
   adc0_tdata       => adc0_tdata,  
   adc0_tkeep       => adc0_tkeep,  
   adc0_tlast       => adc0_tlast,  
   adc0_tready      => adc0_tready,  
   adc0_tstrb       => adc0_tstrb,  
   adc0_tuser       => adc0_tuser,  
   adc0_tvalid      => adc0_tvalid,  
   -- external signals
   fmc_to_cpld      => fmc_to_cpld,
   front_io_fmc     => front_io_fmc,
   clk_to_fpga_p    => clk_to_fpga_p,
   clk_to_fpga_n    => clk_to_fpga_n,
   ext_trigger_p    => ext_trigger_p,
   ext_trigger_n    => ext_trigger_n,
   sync_from_fpga_p => sync_from_fpga_p,
   sync_from_fpga_n => sync_from_fpga_n,
   adc0_clka_p      => adc0_clka_p,
   adc0_clka_n      => adc0_clka_n,
   adc0_da_p        => adc0_da_p,
   adc0_da_n        => adc0_da_n,
   adc0_ovra_p      => adc0_ovra_p,
   adc0_ovra_n      => adc0_ovra_n,
   adc0_db_p        => adc0_db_p,
   adc0_db_n        => adc0_db_n,
   adc0_ovrb_p      => adc0_ovrb_p,
   adc0_ovrb_n      => adc0_ovrb_n,
   adc1_clka_p      => adc1_clka_p,
   adc1_clka_n      => adc1_clka_n,
   adc1_da_p        => adc1_da_p,
   adc1_da_n        => adc1_da_n,
   adc1_ovra_p      => adc1_ovra_p,
   adc1_ovra_n      => adc1_ovra_n,
   dac_sync_p       => dac_sync_p,
   dac_sync_n       => dac_sync_n,
   dac0_dclk_p      => dac0_dclk_p,
   dac0_dclk_n      => dac0_dclk_n,
   dac0_data_p      => dac0_data_p,
   dac0_data_n      => dac0_data_n,
   dac1_dclk_p      => dac1_dclk_p,
   dac1_dclk_n      => dac1_dclk_n,
   dac1_data_p      => dac1_data_p,
   dac1_data_n      => dac1_data_n,
   pg_m2c           => '0',
   prsnt_m2c_l      => '1'       
);

-------------------------------------------------------------------------
-- FMC110 ADC0 Data Recv 
--------------------------------------------------------------------------
-- adc0_tdata,  
-- adc0_tlast,  
adc0_tready  <= '1'; 
adc0_tstrb   <= '0'; 
adc0_tkeep   <= (others=>'0');  
-- adc0_tuser,  
-- adc0_tvalid,

process(axi_aclk)
begin
    if rising_edge(axi_aclk) then

        if adc0_tvalid = '1' then
            adc0_count <= adc0_count + 1; 
        end if;
    end if;
end process;

-------------------------------------------------------------------------
-- FMC110 DAC0 Data Generator 
---------------------------------------------------------------------------
inst0_axi_stream_send:
entity xil_defaultlib.axi_stream_send 
generic map(
    COUNT => x"00000040"
)
port map (
    clk        => axi_aclk,
    rstn       => axi_aresetn,
    tdata      => dac0_tdata,
    tvalid     => dac0_tvalid,
    enable     => dac0_enable 
);


dac0_enable <= '1' when axi_wdata = x"00000008" and axi_awaddr = x"00002404"  and axi_awvalid = '1' and axi_wvalid = '1' else 
                '0';

dac0_tuser <= (others=>'0');
dac0_tlast <= '0'; 



-------------------------------------------------------------------------
-- FMC110 Model 
---------------------------------------------------------------------------  
fmc110_model_inst: 
entity xil_defaultlib.fmc110_model
generic map (
  CLOCK_PERIOD  => 1 ns
)
port map (
  i2c_scl          => i2c_scl,
  i2c_sda          => i2c_sda,
  fmc_to_cpld      => fmc_to_cpld,
  front_io_fmc     => front_io_fmc,
  clk_to_fpga_p    => clk_to_fpga_p,
  clk_to_fpga_n    => clk_to_fpga_n,
  ext_trigger_p    => ext_trigger_p,
  ext_trigger_n    => ext_trigger_n,
  sync_from_fpga_p => sync_from_fpga_p,
  sync_from_fpga_n => sync_from_fpga_n,
  dac_sync_p       => dac_sync_p,
  dac_sync_n       => dac_sync_n,
  dac0_data_p      => dac0_data_p,
  dac0_data_n      => dac0_data_n,
  dac0_dclk_p      => dac0_dclk_p,
  dac0_dclk_n      => dac0_dclk_n,
  dac1_data_p      => dac1_data_p,
  dac1_data_n      => dac1_data_n,
  dac1_dclk_p      => dac1_dclk_p,
  dac1_dclk_n      => dac1_dclk_n,
  adc0_da_p        => adc0_da_p,
  adc0_da_n        => adc0_da_n,
  adc0_ovra_p      => adc0_ovra_p,
  adc0_ovra_n      => adc0_ovra_n,
  adc0_db_p        => adc0_db_p,
  adc0_db_n        => adc0_db_n,
  adc0_ovrb_p      => adc0_ovrb_p,
  adc0_ovrb_n      => adc0_ovrb_n,
  adc0_clka_p      => adc0_clka_p,
  adc0_clka_n      => adc0_clka_n,
  adc1_da_p        => adc1_da_p,
  adc1_da_n        => adc1_da_n,
  adc1_ovra_p      => adc1_ovra_p,
  adc1_ovra_n      => adc1_ovra_n,
  adc1_clka_p      => adc1_clka_p,
  adc1_clka_n      => adc1_clka_n
);

-------------------------------------------------------------------------
-- AXI-Lite Traffic Generator
---------------------------------------------------------------------------  
inst_traffic_gen: 
entity xil_defaultlib.axi_traffic_gen_0
PORT MAP (
  s_axi_aclk             => axi_aclk,
  s_axi_aresetn          => axi_aresetn,
  m_axi_lite_ch1_awaddr  => axi_awaddr,
  m_axi_lite_ch1_awprot  => open,
  m_axi_lite_ch1_awvalid => axi_awvalid,
  m_axi_lite_ch1_awready => axi_awready,
  m_axi_lite_ch1_wdata   => axi_wdata,
  m_axi_lite_ch1_wstrb   => axi_wstrb,
  m_axi_lite_ch1_wvalid  => axi_wvalid,
  m_axi_lite_ch1_wready  => axi_wready,
  m_axi_lite_ch1_bresp   => axi_bresp,
  m_axi_lite_ch1_bvalid  => axi_bvalid,
  m_axi_lite_ch1_bready  => axi_bready,
  m_axi_lite_ch1_araddr  => axi_araddr,
  m_axi_lite_ch1_arvalid => axi_arvalid,
  m_axi_lite_ch1_arready => axi_arready,
  m_axi_lite_ch1_rdata   => axi_rdata,
  m_axi_lite_ch1_rvalid  => axi_rvalid,
  m_axi_lite_ch1_rresp   => axi_rresp,
  m_axi_lite_ch1_rready  => axi_rready,
  done                   => done,
  status                 => status
);

--***********************************************************************************
end architecture Behavioral;
--***********************************************************************************

