--------------------------------------------------------------------------------
-- file name : fmc110_model.vhd
--
-- author    : P. Kortekaas
--
-- company   : 4dsp
--
-- item      : number
--
-- language  : vhdl
--
--------------------------------------------------------------------------------
-- description
-- ===========
--
--
-- notes:
--------------------------------------------------------------------------------
--
--  disclaimer: limited warranty and disclaimer. these designs are
--              provided to you as is.  4dsp specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. 4dsp does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the designs
--              will be corrected. furthermore, 4dsp does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              limitation of liability. in no event will 4dsp or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. this limitation will apply even if 4dsp
--              has been advised of the possibility of such damage. this
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--      from
-- ver  pcb mod    date      changes
-- ===  =======    ========  =======
--
-- 0.0    0        05-12-2006        new version
--
----------------------------------------------
--
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Specify libraries
--------------------------------------------------------------------------------
library ieee;
  use ieee.std_logic_unsigned.all;
  use ieee.std_logic_misc.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity declaration
--------------------------------------------------------------------------------
entity fmc110_model is
generic (
  CLOCK_PERIOD  : time := 1 ns
);
port (
  i2c_scl          : inout std_logic;
  i2c_sda          : inout std_logic;
  fmc_to_cpld      : inout std_logic_vector(3 downto 0);
  front_io_fmc     : inout std_logic_vector(3 downto 0);
  clk_to_fpga_p    : out   std_logic;
  clk_to_fpga_n    : out   std_logic;
  ext_trigger_p    : out   std_logic;
  ext_trigger_n    : out   std_logic;
  sync_from_fpga_p : in    std_logic;
  sync_from_fpga_n : in    std_logic;
  dac_sync_p       : in    std_logic;
  dac_sync_n       : in    std_logic;
  dac0_data_p      : in    std_logic_vector(15 downto 0);
  dac0_data_n      : in    std_logic_vector(15 downto 0);
  dac0_dclk_p      : in    std_logic;
  dac0_dclk_n      : in    std_logic;
  dac1_data_p      : in    std_logic_vector(15 downto 0);
  dac1_data_n      : in    std_logic_vector(15 downto 0);
  dac1_dclk_p      : in    std_logic;
  dac1_dclk_n      : in    std_logic;
  adc0_da_p        : out   std_logic_vector(11 downto 0);
  adc0_da_n        : out   std_logic_vector(11 downto 0);
  adc0_ovra_p      : out   std_logic;
  adc0_ovra_n      : out   std_logic;
  adc0_db_p        : out   std_logic_vector(11 downto 0);
  adc0_db_n        : out   std_logic_vector(11 downto 0);
  adc0_ovrb_p      : out   std_logic;
  adc0_ovrb_n      : out   std_logic;
  adc0_clka_p      : out   std_logic;
  adc0_clka_n      : out   std_logic;
  adc1_da_p        : out   std_logic_vector(11 downto 0);
  adc1_da_n        : out   std_logic_vector(11 downto 0);
  adc1_ovra_p      : out   std_logic;
  adc1_ovra_n      : out   std_logic;
  adc1_clka_p      : out   std_logic;
  adc1_clka_n      : out   std_logic
);
end entity fmc110_model;

--------------------------------------------------------------------------------
-- Architecture declaration
--------------------------------------------------------------------------------
architecture fmc110_model_beh of fmc110_model is

--------------------------------------------------------------------------------
-- Constant declarations
--------------------------------------------------------------------------------

-----------------------------------------------------------------------------------
-- Signal declarations
-----------------------------------------------------------------------------------
signal spi_sdio        : std_logic;
signal clk_n_reset     : std_logic;
signal vm_n_int        : std_logic;

signal clock           : std_logic := '0';
signal clk_to_fpga     : std_logic := '0';
signal clk_to_adc      : std_logic := '0';
signal clk_to_dac      : std_logic := '0';

signal clk_to_adc_div2 : std_logic := '0';
signal clk_to_adc_div4 : std_logic := '0';

signal adc0_clka       : std_logic := '0';
signal adc0_clkb       : std_logic := '0';
signal adc0_d_p        : std_logic;
signal adc0_d_n        : std_logic;
signal prbs0           : std_logic_vector(7 downto 1) := "0000001";

signal adc1_clka       : std_logic := '0';
signal adc1_d_p        : std_logic;
signal adc1_d_n        : std_logic;
signal prbs1           : std_logic_vector(7 downto 1) := "0000001";

-----------------------------------------------------------------------------------
-- Component declarations
-----------------------------------------------------------------------------------
component i2c_slave_model is
generic (
  I2C_ADR : std_logic_vector(6 downto 0) := "1001010"
);
port (
  scl : in std_logic;
  sda : inout std_logic
);
end component;

component fmc110_cpld is
port (
  fmc_to_cpld     : inout std_logic_vector(3 downto 0); -- SPI communication with host
  spi_sclk        : out   std_logic;                    -- Local SPI clock
  spi_sdio        : inout std_logic;                    -- Local SPI data in/out
  spi_n_cs        : out   std_logic_vector(4 downto 0); -- Local SPI chip selects
  clk_n_reset     : out   std_logic;                    -- SPI reset Clock Tree
  dac_n_reset     : out   std_logic;                    -- SPI reset DAC
  clksrc_sel      : out   std_logic_vector(2 downto 0); -- Clock source selection
  syncsrc_sel     : out   std_logic_vector(1 downto 0); -- Synchronisation source selection
  clk_refmon      : in    std_logic;                    -- Clock Tree reference monitor status
  clk_ld          : in    std_logic;                    -- Clock Tree PLL lock status
  clk_status      : in    std_logic;                    -- Clock Tree programmable status
  vm_n_int        : in    std_logic;                    -- Voltage Monitor alarm interupt
  ps_en           : in    std_logic;                    -- Local Power Supply Enable
  led             : out   std_logic;                    -- Led driver
  eeprom_wp       : out   std_logic;                    -- EEPROM write protect
  front_io_dir    : out   std_logic_vector(3 downto 0); -- Front panel LVTTL IO direction control
  mictor_io       : inout std_logic_vector(3 downto 0); -- Auxilary IO on Mictor connectors
  fan_n_en        : out   std_logic_vector(3 downto 0)  -- Local FAN enable controls
);
end component fmc110_cpld;

begin

--------------------------------------------------------------------------------
-- CPLD
--------------------------------------------------------------------------------
fmc110_cpld_inst : fmc110_cpld
port map (
  fmc_to_cpld  =>  fmc_to_cpld,  -- SPI communication with host
  spi_sclk     =>  open,        -- Local SPI clock
  spi_sdio     =>  spi_sdio,    -- Local SPI data in/out
  spi_n_cs     =>  open,        -- Local SPI chip selects
  clk_n_reset  =>  clk_n_reset, -- SPI reset Clock Tree
  dac_n_reset  =>  open,        -- SPI reset DAC
  clksrc_sel   =>  open,        -- Clock source selection
  syncsrc_sel  =>  open,        -- Synchronisation source selection
  clk_refmon   =>  '0',         -- Clock Tree reference monitor status
  clk_ld       =>  '0',         -- Clock Tree PLL lock status
  clk_status   =>  '0',         -- Clock Tree programmable status
  vm_n_int     =>  vm_n_int,    -- Voltage Monitor alarm interupt
  ps_en        =>  '1',         -- Local Power Supply Enable
  led          =>  open,        -- Led driver
  eeprom_wp    =>  open,        -- EEPROM write protect
  front_io_dir =>  open,        -- Front panel LVTTL IO direction control
  mictor_io    =>  open,        -- Auxilary IO on Mictor connectors
  fan_n_en     =>  open         -- Local FAN enable controls
);

spi_sdio <= 'H';

clock <= not clock after CLOCK_PERIOD / 2;

clk_to_fpga <= clock when clk_n_reset = '1' else '0';
clk_to_adc  <= clock when clk_n_reset = '1' else '0';
clk_to_dac  <= clock when clk_n_reset = '1' else '0';

--LVDS output
clk_to_fpga_p <= clk_to_fpga;
clk_to_fpga_n <= not clk_to_fpga;

--------------------------------------------------------------------------------
-- ADC model
--------------------------------------------------------------------------------

process (clk_to_adc)
begin
  if (rising_edge(clk_to_adc)) then
    clk_to_adc_div2 <= not clk_to_adc_div2;
  end if;
end process;

process (clk_to_adc_div2)
begin
  if (rising_edge(clk_to_adc_div2)) then
    clk_to_adc_div4 <= not clk_to_adc_div4;
  end if;
end process;

-- ADC0 / 2-bus
adc0_clka   <= clk_to_adc_div4;
adc0_clka_p <=     adc0_clka;
adc0_clka_n <= not adc0_clka;

adc0_ovra_p <= sync_from_fpga_p;
adc0_ovra_n <= sync_from_fpga_n;

adc0_ovrb_p <= sync_from_fpga_p;
adc0_ovrb_n <= sync_from_fpga_n;

process (clk_to_adc_div2)
begin
  if (rising_edge(clk_to_adc_div2)) then
    prbs0 <= prbs0(6 downto 1) & (prbs0(7) XOR prbs0(6));
  end if;
end process;

adc0_d_p <=     prbs0(7);
adc0_d_n <= not prbs0(7);
adc0_da_p   <= (others => adc0_d_p);
adc0_da_n   <= (others => adc0_d_n);
adc0_db_p   <= (others => adc0_d_p);
adc0_db_n   <= (others => adc0_d_n);

-- ADC1 / 1-bus
adc1_clka   <= clk_to_adc_div2;
adc1_clka_p <=     adc1_clka;
adc1_clka_n <= not adc1_clka;

adc1_ovra_p <= sync_from_fpga_p;
adc1_ovra_n <= sync_from_fpga_n;

process (clk_to_adc)
begin
  if (rising_edge(clk_to_adc)) then
    prbs1 <= prbs1(6 downto 1) & (prbs1(7) XOR prbs1(6));
  end if;
end process;

adc1_d_p <=     prbs1(7);
adc1_d_n <= not prbs1(7);

adc1_da_p(0)  <= transport adc1_d_p after 1000 ps;
adc1_da_n(0)  <= transport adc1_d_n after 1000 ps;

adc1_da_p(1)  <= transport adc1_d_p after 2100 ps;
adc1_da_n(1)  <= transport adc1_d_n after 2100 ps;

adc1_da_p(2)  <= transport adc1_d_p after 1060 ps;
adc1_da_n(2)  <= transport adc1_d_n after 1060 ps;

adc1_da_p(3)  <= transport adc1_d_p after 1780 ps;
adc1_da_n(3)  <= transport adc1_d_n after 1780 ps;

adc1_da_p(4)  <= transport adc1_d_p after 1120 ps;
adc1_da_n(4)  <= transport adc1_d_n after 1120 ps;

adc1_da_p(5)  <= transport adc1_d_p after 2110 ps;
adc1_da_n(5)  <= transport adc1_d_n after 2110 ps;

adc1_da_p(6)  <= transport adc1_d_p after 1200 ps;
adc1_da_n(6)  <= transport adc1_d_n after 1200 ps;

adc1_da_p(7)  <= transport adc1_d_p after 1230 ps;
adc1_da_n(7)  <= transport adc1_d_n after 1230 ps;

adc1_da_p(8)  <= transport adc1_d_p after 1160 ps;
adc1_da_n(8)  <= transport adc1_d_n after 1160 ps;

adc1_da_p(9)  <= transport adc1_d_p after 1020 ps;
adc1_da_n(9)  <= transport adc1_d_n after 1020 ps;

adc1_da_p(10) <= transport adc1_d_p after 1000 ps;
adc1_da_n(10) <= transport adc1_d_n after 1000 ps;

adc1_da_p(11) <= transport adc1_d_p after 1130 ps;
adc1_da_n(11) <= transport adc1_d_n after 1130 ps;

--------------------------------------------------------------------------------
-- Trigger model
--------------------------------------------------------------------------------
process
begin
  ext_trigger_p <= '0';
  ext_trigger_n <= '1';
  wait for 80 us;
  ext_trigger_p <= '1';
  ext_trigger_n <= '0';
  wait for 20 us;
  ext_trigger_p <= '0';
  ext_trigger_n <= '1';
  wait;
end process;

--------------------------------------------------------------------------------
-- Outputs
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Monitoring
--------------------------------------------------------------------------------
adt7411: i2c_slave_model
generic map (
  I2C_ADR => "1010111"
)
port map (
  scl => i2c_scl,
  sda => i2c_sda
);

i2c_scl  <= 'H';
i2c_sda  <= 'H';
vm_n_int <= 'H';

--------------------------------------------------------------------------------
-- End
--------------------------------------------------------------------------------
end architecture fmc110_model_beh;

