-------------------------------------------------------------------------------------
-- FILE NAME : dac5681z_phy.vhd
--
-- AUTHOR    : Peter Kortekaas
--
-- COMPANY   : 4DSP
--
-- ITEM      : 1
--
-- UNITS     : Entity       - dac5681z_phy
--             architecture - dac5681z_phy_syn
--
-- LANGUAGE  : VHDL
--
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
-- DESCRIPTION
-- ===========
--
-- dac5681z_phy
-- Notes: dac5681z_phy
-------------------------------------------------------------------------------------
--  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--              provided to you as is.  4DSP specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. 4DSP does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, 4DSP does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will 4DSP or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if 4DSP
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
----------------------------------------------

-- Library declarations
library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;
library unisim;
  use unisim.vcomponents.all;

entity dac5681z_phy is
  generic
  (
    START_ADDR   : std_logic_vector(27 downto 0) := x"0000000";
    STOP_ADDR    : std_logic_vector(27 downto 0) := x"00000FF"
  );
  port (

    -- Global signals
    rst          : in  std_logic;

    -- Command Interface
    clk_cmd      : in  std_logic;
    in_cmd_val   : in  std_logic;
    in_cmd       : in  std_logic_vector(63 downto 0);
    out_cmd_val  : out std_logic;
    out_cmd      : out std_logic_vector(63 downto 0);
    in_cmd_busy  : out std_logic;

    -- Input data
    phy_in_clk   : out std_logic;                      -- clock 1/8 of the sample frequecy
    phy_in_dval  : in  std_logic;                      -- valid signal
    phy_in_data  : in  std_logic_vector(127 downto 0);  -- 8 samples in parallel in 16-bit format

    -- DAC interface
    clk_to_fpga  : in  std_logic;

    dac_dclk_p   : out std_logic;
    dac_dclk_n   : out std_logic;
    dac_data_p   : out std_logic_vector(15 downto 0);
    dac_data_n   : out std_logic_vector(15 downto 0);
    dac_sync     : out std_logic

  );
end dac5681z_phy;

architecture dac5681z_phy_syn of dac5681z_phy is

component fmc110_stellar_cmd is
generic
(
  START_ADDR           : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR            : std_logic_vector(27 downto 0) := x"00000FF"
);
port
(
  reset                : in  std_logic;
  -- Command Interface
  clk_cmd              : in  std_logic;                     --cmd_in and cmd_out are synchronous to this clock;
  out_cmd              : out std_logic_vector(63 downto 0);
  out_cmd_val          : out std_logic;
  in_cmd               : in  std_logic_vector(63 downto 0);
  in_cmd_val           : in  std_logic;
  -- Register interface
  clk_reg              : in  std_logic;                     --register interface is synchronous to this clock
  out_reg              : out std_logic_vector(31 downto 0); --caries the out register data
  out_reg_val          : out std_logic;                     --the out_reg has valid data  (pulse)
  out_reg_addr         : out std_logic_vector(27 downto 0); --out register address
  in_reg               : in  std_logic_vector(31 downto 0); --requested register data is placed on this bus
  in_reg_val           : in  std_logic;                     --pulse to indicate requested register is valid
  in_reg_req           : out std_logic;                     --pulse to request data
  in_reg_addr          : out std_logic_vector(27 downto 0);  --requested address
  --mailbox interface
  mbx_in_reg           : in  std_logic_vector(31 downto 0); --value of the mailbox to send
  mbx_in_val           : in  std_logic                      --pulse to indicate mailbox is valid
);
end component fmc110_stellar_cmd;

component dac_mmcm is
port (
  -- Clock in ports
  clk_in1  : in  std_logic;
  -- Clock out ports
  clk_out1 : out std_logic;
  clk_out2 : out std_logic;
  clk_out3 : out std_logic;
  -- Status and control signals
  reset    : in  std_logic;
  locked   : out std_logic
);
end component dac_mmcm;

component dac5681z_phy_fifo is
	port (
	  rst           : in std_logic;
	  wr_clk        : in std_logic;
	  rd_clk        : in std_logic;
	  din           : in std_logic_vector(127 downto 0);
	  wr_en         : in std_logic;
	  rd_en         : in std_logic;
	  dout          : out std_logic_vector(63 downto 0);
	  full          : out std_logic;
	  empty         : out std_logic;
	  valid         : out std_logic
  );
end component;

----------------------------------------------------------------------------------------------------
-- Constants
----------------------------------------------------------------------------------------------------
constant ADDR_COMMAND : std_logic_vector(31 downto 0) := x"00000000";
constant ADDR_CONTROL : std_logic_vector(31 downto 0) := x"00000001";
constant ADDR_DLY_INC : std_logic_vector(31 downto 0) := x"00000002";
constant ADDR_DLY_DEC : std_logic_vector(31 downto 0) := x"00000003";

type odly_array is array(15 downto 0) of integer;
constant ODLY_DATA : odly_array := (1,0,0,0,0,2,1,2,0,0,0,1,1,1,2,2);
constant ODLY_CLK  : integer := 0;

component pulse2pulse is
port (
  in_clk   : in  std_logic;
  out_clk  : in  std_logic;
  rst      : in  std_logic;
  pulsein  : in  std_logic;
  inbusy   : out std_logic;
  pulseout : out std_logic
);
end component pulse2pulse;

----------------------------------------------------------------------------------------------------
-- Signals
----------------------------------------------------------------------------------------------------
signal txclkdiv2       : std_logic; --Fs/2
signal txclkdiv4       : std_logic; --Fs/4
signal txclkdiv8       : std_logic; --Fs/8

signal dac_dclk_predly : std_logic;
signal dac_dclk_prebuf : std_logic;

signal dac_data_predly : std_logic_vector(15 downto 0);
signal dac_data_prebuf : std_logic_vector(15 downto 0);

signal out_reg_val     : std_logic;
signal out_reg_addr    : std_logic_vector(27 downto 0);
signal out_reg         : std_logic_vector(31 downto 0);

signal in_reg_req      : std_logic;
signal in_reg_addr     : std_logic_vector(27 downto 0);
signal in_reg_val      : std_logic;
signal in_reg          : std_logic_vector(31 downto 0);

signal io_rst          : std_logic;
signal io_rst_buf      : std_logic;

signal training_en     : std_logic;
signal force_dcm_rst   : std_logic;
signal dcm_rst         : std_logic;
signal sync_txen       : std_logic;

signal odly_inc        : std_logic_vector(15 downto 0);
signal odly_dec        : std_logic_vector(15 downto 0);
signal odly_ce         : std_logic_vector(15 downto 0);

signal dcm_locked      : std_logic;
signal oserdes_rst     : std_logic;

signal fifo_in_val     : std_logic;
signal fifo_in         : std_logic_vector(127 downto 0);
signal fifo_out        : std_logic_vector(63 downto 0);
signal fifo_out_val    : std_logic;
signal odata           : std_logic_vector(63 downto 0);
signal odata_mux       : std_logic_vector(63 downto 0);
signal odata_reg       : std_logic_vector(63 downto 0);

begin

----------------------------------------------------------------------------------------------------
-- Stellar Command Interface
----------------------------------------------------------------------------------------------------
fmc110_stellar_cmd_inst : fmc110_stellar_cmd
generic map
(
  START_ADDR   => START_ADDR,
  STOP_ADDR    => STOP_ADDR
)
port map
(
  reset        => rst,

  clk_cmd      => clk_cmd,
  in_cmd_val   => in_cmd_val,
  in_cmd       => in_cmd,
  out_cmd_val  => out_cmd_val,
  out_cmd      => out_cmd,

  clk_reg      => clk_cmd,
  out_reg_val  => out_reg_val,
  out_reg_addr => out_reg_addr,
  out_reg      => out_reg,

  in_reg_req   => in_reg_req,
  in_reg_addr  => in_reg_addr,
  in_reg_val   => in_reg_val,
  in_reg       => in_reg,

  mbx_in_val   => '0',
  mbx_in_reg   => (others => '0')
);

in_cmd_busy <= '0';

----------------------------------------------------------------------------------------------------
-- Registers
----------------------------------------------------------------------------------------------------
process (rst, clk_cmd)
begin
  if (rst = '1') then

    io_rst        <= '1';
    training_en   <= '0';
    force_dcm_rst <= '0';
    sync_txen     <= '0';
    odly_inc      <= (others => '0');
    odly_dec      <= (others => '0');
    in_reg_val    <= '0';
    in_reg        <= (others => '0');

  elsif (rising_edge(clk_cmd)) then

    -- Write controls
    if (out_reg_val = '1' and out_reg_addr = ADDR_COMMAND) then
      io_rst <= out_reg(0);
    else
      io_rst <= '0';
    end if;

    if (out_reg_val = '1' and out_reg_addr = ADDR_CONTROL) then
      training_en   <= out_reg(0);
      force_dcm_rst <= out_reg(1);
      sync_txen     <= out_reg(2);
    end if;

    -- ODELAY Increment
    if (out_reg_val = '1' and out_reg_addr = ADDR_DLY_INC) then
      odly_inc <= out_reg(15 downto 0);
    else
      odly_inc <= (others => '0');
    end if;

    -- ODELAY Decrement
    if (out_reg_val = '1' and out_reg_addr = ADDR_DLY_DEC) then
      odly_dec <= out_reg(15 downto 0);
    else
      odly_dec <= (others => '0');
    end if;

    -- Read
    if (in_reg_req = '1' and in_reg_addr = ADDR_CONTROL) then
      in_reg_val <= '1';
      in_reg     <= conv_std_logic_vector(0, 32-3) & sync_txen & force_dcm_rst & training_en;
    else
      in_reg_val <= '0';
      in_reg     <= in_reg;
    end if;

  end if;
end process;

----------------------------------------------------------------------------------------------------
--Data clock input (Fs/2)
----------------------------------------------------------------------------------------------------
dac_mmcm_inst : dac_mmcm
port map (
  clk_in1  => clk_to_fpga, --500MHz
  clk_out1 => txclkdiv2,
  clk_out2 => txclkdiv4, --250MHz
  clk_out3 => txclkdiv8, --125MHz
  reset    => dcm_rst,
  locked   => dcm_locked
);

dcm_rst <= rst or force_dcm_rst;

----------------------------------------------------------------------------------------------------
--Oserdes reset generation
----------------------------------------------------------------------------------------------------
pulse2pulse_inst : pulse2pulse
port map (
  in_clk   => clk_cmd,
  out_clk  => txclkdiv4,
  rst      => rst,
  pulsein  => io_rst,
  inbusy   => open,
  pulseout => io_rst_buf
);

process (txclkdiv4)
begin
  if (rst = '1') then

    oserdes_rst <= '1';

  elsif (rising_edge(txclkdiv4)) then

    if (io_rst_buf = '1' or dcm_locked = '0') then
      oserdes_rst <= '1';
    else
      oserdes_rst <= '0';
    end if;

  end if;
end process;

----------------------------------------------------------------------------------------------------
--Output serdes and LVDS buffers for clock
----------------------------------------------------------------------------------------------------

  --master oserdes in clock path
  oserdes_inst : oserdes
  generic map (
    DATA_RATE_OQ => "DDR",
    DATA_RATE_TQ => "DDR",
    DATA_WIDTH => 4,
    INIT_OQ => '0',
    INIT_TQ => '0',
    SERDES_MODE => "MASTER",
    SRVAL_OQ => '0',
    SRVAL_TQ => '0',
    TRISTATE_WIDTH => 1
  )
  port map (
    oq        => dac_dclk_prebuf,
    shiftout1 => open,
    shiftout2 => open,
    tq        => open,
    clk       => txclkdiv2,
    clkdiv    => txclkdiv4,
    d1        => '1',
    d2        => '0',
    d3        => '1',
    d4        => '0',
    d5        => '0',
    d6        => '0',
    oce       => '1',
    rev       => '0',
    shiftin1  => '0',
    shiftin2  => '0',
    sr        => oserdes_rst,
    t1        => '0',
    t2        => '0',
    t3        => '0',
    t4        => '0',
    tce       => '0'
  );

	--output buffers
  obufds_inst : obufds
  port map (
    i  => dac_dclk_prebuf,
    o  => dac_dclk_p,
    ob => dac_dclk_n
  );

----------------------------------------------------------------------------------------------------
--Output serdes and LVDS buffers
----------------------------------------------------------------------------------------------------
ch_logic: for i in 0 to 15 generate

  --master oserdes in data path
  oserdes_inst : oserdes
  generic map (
    DATA_RATE_OQ => "DDR",
    DATA_RATE_TQ => "DDR",
    DATA_WIDTH => 4,
    INIT_OQ => '0',
    INIT_TQ => '0',
    SERDES_MODE => "MASTER",
    SRVAL_OQ => '0',
    SRVAL_TQ => '0',
    TRISTATE_WIDTH => 1
  )
  port map (
    oq        => dac_data_predly(i),
    shiftout1 => open,
    shiftout2 => open,
    tq        => open,
    clk       => txclkdiv2,
    clkdiv    => txclkdiv4,
    d1        => odata_reg(i*4+0),
    d2        => odata_reg(i*4+1),
    d3        => odata_reg(i*4+2),
    d4        => odata_reg(i*4+3),
    d5        => '0',
    d6        => '0',
    oce       => '1',
    rev       => '0',
    shiftin1  => '0',
    shiftin2  => '0',
    sr        => oserdes_rst,
    t1        => '0',
    t2        => '0',
    t3        => '0',
    t4        => '0',
    tce       => '0'
  );

  odly_ce(i) <= odly_inc(i) or odly_dec(i);

  iodelay_inst : iodelay
  generic map (
    IDELAY_TYPE	=> "FIXED",
    IDELAY_VALUE => 0,
    ODELAY_VALUE => ODLY_DATA(i),
    HIGH_PERFORMANCE_MODE => TRUE,
    SIGNAL_PATTERN => "DATA",
    REFCLK_FREQUENCY => 200.0,
    DELAY_SRC => "O"
  )
  port map (
    dataout => dac_data_prebuf(i),
    idatain => '0',
    odatain => dac_data_predly(i),
    datain => '0',
    --clkin => '0',
    t => '0',
    ce => '0',--odly_ce(i),
    inc => '0',--odly_inc(i),
    --cntvaluein => (others => '0'),
    --cinvctrl => '0'
    c => '0',--clk_cmd,
    rst => io_rst
  );

	--output buffers
  obufds_inst : obufds
  port map (
    i  => dac_data_predly(i),--dac_data_prebuf(i),
    o  => dac_data_n(i),
    ob => dac_data_p(i)
  );

end generate;

----------------------------------------------------------------------------------------------------
--data source: training pattern or data
----------------------------------------------------------------------------------------------------
process (txclkdiv8)
begin
  if (rising_edge(txclkdiv8)) then

    if (training_en = '1') then
      fifo_in_val <= '1';
      fifo_in     <= x"5555AAAA5555AAAA5555AAAA5555AAAA";
    else
      fifo_in_val <= phy_in_dval;
      fifo_in     <= phy_in_data(63 downto 0) & phy_in_data(127 downto 64);
    end if;

  end if;
end process;

----------------------------------------------------------------------------------------------------
-- Input FIFO - 128bit @ Fs/8 to 64bit @ Fs/4
----------------------------------------------------------------------------------------------------
dac5681z_phy_fifo_inst : dac5681z_phy_fifo
port map (
  rst    => rst,
  wr_clk => txclkdiv8,
  rd_clk => txclkdiv4,
  din    => fifo_in,
  wr_en  => fifo_in_val,
  rd_en  => '1',
  dout   => fifo_out,
  full   => open,
  empty  => open,
  valid  => fifo_out_val
);

----------------------------------------------------------------------------------------------------
-- Reorder FIFO output
----------------------------------------------------------------------------------------------------
FIFO_SAMPLES: for s in 0 to 3 generate

  FIFO_BITS: for b in 0 to 15 generate

    odata(b*4+s) <= fifo_out(s*16+b);

  end generate;

end generate;

process (txclkdiv4)
begin
  if (rst = '1') then

    odata_mux <= (others => '0');
    odata_reg <= (others => '0');

  elsif (txclkdiv4'event and txclkdiv4 = '1') then

    if (fifo_out_val = '1') then
      odata_mux <= not odata; --inverted, will be inverted again by P/N swap on output
    else
      odata_mux <= (others => '0');
    end if;

    odata_reg <= odata_mux;

  end if;
end process;

----------------------------------------------------------------------------------------------------
-- Connect entity
----------------------------------------------------------------------------------------------------
phy_in_clk <= txclkdiv8;
dac_sync   <= sync_txen;

----------------------------------------------------------------------------------------------------
-- end
----------------------------------------------------------------------------------------------------
end dac5681z_phy_syn;
