-------------------------------------------------------------------------------------
-- FILE NAME : fmc110_if.vhd
--
-- AUTHOR    : Peter Kortekaas
--
-- COMPANY   : 4DSP
--
-- ITEM      : 1
--
-- UNITS     : Entity       - fmc110_if
--             architecture - fmc110_if_syn
--
-- LANGUAGE  : VHDL
--
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
-- DESCRIPTION
-- ===========
--
-- fmc110_if
-- Notes: fmc110_if
-------------------------------------------------------------------------------------
--  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--              provided to you as is.  4DSP specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. 4DSP does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, 4DSP does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will 4DSP or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if 4DSP
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
----------------------------------------------

-- Library declarations
library ieee;
  use ieee.std_logic_unsigned.all;
  use ieee.std_logic_misc.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_1164.all;
library unisim;
  use unisim.vcomponents.all;
library work;

entity fmc110_if is
generic (
  START_ADDR       : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR        : std_logic_vector(27 downto 0) := x"00000FF"
);
port (
  -- Global signals
  rst              : in    std_logic;
  clk              : in    std_logic;

  -- Command Interface
  clk_cmd          : in    std_logic;
  in_cmd_val       : in    std_logic;
  in_cmd           : in    std_logic_vector(63 downto 0);
  out_cmd_val      : out   std_logic;
  out_cmd          : out   std_logic_vector(63 downto 0);
  cmd_busy         : out   std_logic;

  --Output ports for ADC data
  adc0_out_stop    : in    std_logic;
  adc0_out_dval    : out   std_logic;
  adc0_out_data    : out   std_logic_vector(63 downto 0);
  adc1_out_stop    : in    std_logic;
  adc1_out_dval    : out   std_logic;
  adc1_out_data    : out   std_logic_vector(63 downto 0);

  --Input ports for DAC data
  dac0_in_stop     : out   std_logic;
  dac0_in_dval     : in    std_logic;
  dac0_in_data     : in    std_logic_vector(63 downto 0);
  dac1_in_stop     : out   std_logic;
  dac1_in_dval     : in    std_logic;
  dac1_in_data     : in    std_logic_vector(63 downto 0);

  --External signals
  fmc_to_cpld      : inout std_logic_vector(3 downto 0);
  front_io_fmc     : inout std_logic_vector(3 downto 0);

  clk_to_fpga_p    : in    std_logic;
  clk_to_fpga_n    : in    std_logic;
  ext_trigger_p    : in    std_logic;
  ext_trigger_n    : in    std_logic;
  sync_from_fpga_p : out   std_logic;
  sync_from_fpga_n : out   std_logic;

  adc0_clka_p      : in    std_logic;
  adc0_clka_n      : in    std_logic;
  adc0_da_p        : in    std_logic_vector(11 downto 0);
  adc0_da_n        : in    std_logic_vector(11 downto 0);
  adc0_ovra_p      : in    std_logic;
  adc0_ovra_n      : in    std_logic;
  adc0_db_p        : in    std_logic_vector(11 downto 0);
  adc0_db_n        : in    std_logic_vector(11 downto 0);
  adc0_ovrb_p      : in    std_logic;
  adc0_ovrb_n      : in    std_logic;

  adc1_clka_p      : in    std_logic;
  adc1_clka_n      : in    std_logic;
  adc1_da_p        : in    std_logic_vector(11 downto 0);
  adc1_da_n        : in    std_logic_vector(11 downto 0);
  adc1_ovra_p      : in    std_logic;
  adc1_ovra_n      : in    std_logic;

  dac_sync_p       : out   std_logic;
  dac_sync_n       : out   std_logic;

  dac0_dclk_p      : out   std_logic;
  dac0_dclk_n      : out   std_logic;
  dac0_data_p      : out   std_logic_vector(15 downto 0);
  dac0_data_n      : out   std_logic_vector(15 downto 0);

  dac1_dclk_p      : out   std_logic;
  dac1_dclk_n      : out   std_logic;
  dac1_data_p      : out   std_logic_vector(15 downto 0);
  dac1_data_n      : out   std_logic_vector(15 downto 0);

  pg_m2c           : in    std_logic;
  prsnt_m2c_l      : in    std_logic

);
end fmc110_if;

architecture fmc110_if_syn of fmc110_if is

----------------------------------------------------------------------------------------------------
-- Constant declaration
----------------------------------------------------------------------------------------------------
constant START_ADDR_FMC110_CTRL    : std_logic_vector(27 downto 0) := START_ADDR + x"0000000";
constant STOP_ADDR_FMC110_CTRL     : std_logic_vector(27 downto 0) := START_ADDR + x"000000F"; --min. range 16#0006

constant START_ADDR_ADS5400_PHY0   : std_logic_vector(27 downto 0) := START_ADDR + x"0000010";
constant STOP_ADDR_ADS5400_PHY0    : std_logic_vector(27 downto 0) := START_ADDR + x"000001F"; --min. range 16#000A

constant START_ADDR_DAC5681Z_PHY0  : std_logic_vector(27 downto 0) := START_ADDR + x"0000020";
constant STOP_ADDR_DAC5681Z_PHY0   : std_logic_vector(27 downto 0) := START_ADDR + x"000002F"; --min. range 16#0010

constant START_ADDR_ADS5400_CTRL0  : std_logic_vector(27 downto 0) := START_ADDR + x"0000100";
constant STOP_ADDR_ADS5400_CTRL0   : std_logic_vector(27 downto 0) := START_ADDR + x"0000120"; --min. range 16#0021

constant START_ADDR_DAC5681Z_CTRL0 : std_logic_vector(27 downto 0) := START_ADDR + x"0000200";
constant STOP_ADDR_DAC5681Z_CTRL0  : std_logic_vector(27 downto 0) := START_ADDR + x"0000210"; --min. range 16#0011

constant START_ADDR_AD9517_CTRL    : std_logic_vector(27 downto 0) := START_ADDR + x"0000300";
constant STOP_ADDR_AD9517_CTRL     : std_logic_vector(27 downto 0) := START_ADDR + x"0000533"; --min. range 16#0234

constant START_ADDR_FREQ_CNT       : std_logic_vector(27 downto 0) := START_ADDR + x"0000600";
constant STOP_ADDR_FREQ_CNT        : std_logic_vector(27 downto 0) := START_ADDR + x"0000601"; --min. range 16#0002

constant START_ADDR_ADS5400_PHY1   : std_logic_vector(27 downto 0) := START_ADDR + x"0000710";
constant STOP_ADDR_ADS5400_PHY1    : std_logic_vector(27 downto 0) := START_ADDR + x"000071F"; --min. range 16#000A

constant START_ADDR_DAC5681Z_PHY1  : std_logic_vector(27 downto 0) := START_ADDR + x"0000720";
constant STOP_ADDR_DAC5681Z_PHY1   : std_logic_vector(27 downto 0) := START_ADDR + x"000072F"; --min. range 16#0010

constant START_ADDR_ADS5400_CTRL1  : std_logic_vector(27 downto 0) := START_ADDR + x"0000800";
constant STOP_ADDR_ADS5400_CTRL1   : std_logic_vector(27 downto 0) := START_ADDR + x"0000820"; --min. range 16#0021

constant START_ADDR_DAC5681Z_CTRL1 : std_logic_vector(27 downto 0) := START_ADDR + x"0000900";
constant STOP_ADDR_DAC5681Z_CTRL1  : std_logic_vector(27 downto 0) := START_ADDR + x"0000910"; --min. range 16#0011

constant START_ADDR_CPLD_CTRL      : std_logic_vector(27 downto 0) := START_ADDR + x"0000920";
constant STOP_ADDR_CPLD_CTRL       : std_logic_vector(27 downto 0) := START_ADDR + x"0000924"; --min. range 16#0010

----------------------------------------------------------------------------------------------------
--CPLD Preselection Bytes
----------------------------------------------------------------------------------------------------
constant PRESEL_CPLD  : std_logic_vector(7 downto 0) := x"00";
constant PRESEL_ADC0  : std_logic_vector(7 downto 0) := x"80";
constant PRESEL_ADC1  : std_logic_vector(7 downto 0) := x"81";
constant PRESEL_DAC0  : std_logic_vector(7 downto 0) := x"82";
constant PRESEL_DAC1  : std_logic_vector(7 downto 0) := x"83";
constant PRESEL_CLK0  : std_logic_vector(7 downto 0) := x"84";

----------------------------------------------------------------------------------------------------
--Signal declaration
----------------------------------------------------------------------------------------------------

signal     out_cmd_val0 : std_logic;
signal         out_cmd0 : std_logic_vector(63 downto 0);
signal        cmd_busy0 : std_logic;

signal     out_cmd_val1 : std_logic;
signal         out_cmd1 : std_logic_vector(63 downto 0);
signal        cmd_busy1 : std_logic;

signal     out_cmd_val2 : std_logic;
signal         out_cmd2 : std_logic_vector(63 downto 0);
signal        cmd_busy2 : std_logic;

signal     out_cmd_val3 : std_logic;
signal         out_cmd3 : std_logic_vector(63 downto 0);
signal        cmd_busy3 : std_logic;

signal     out_cmd_val4 : std_logic;
signal         out_cmd4 : std_logic_vector(63 downto 0);
signal        cmd_busy4 : std_logic;

signal     out_cmd_val5 : std_logic;
signal         out_cmd5 : std_logic_vector(63 downto 0);
signal        cmd_busy5 : std_logic;

signal     out_cmd_val6 : std_logic;
signal         out_cmd6 : std_logic_vector(63 downto 0);
signal        cmd_busy6 : std_logic;

signal     out_cmd_val7 : std_logic;
signal         out_cmd7 : std_logic_vector(63 downto 0);
signal        cmd_busy7 : std_logic;

signal     out_cmd_val8 : std_logic;
signal         out_cmd8 : std_logic_vector(63 downto 0);
signal        cmd_busy8 : std_logic;

signal     out_cmd_val9 : std_logic;
signal         out_cmd9 : std_logic_vector(63 downto 0);
signal        cmd_busy9 : std_logic;

signal    out_cmd_val10 : std_logic;
signal        out_cmd10 : std_logic_vector(63 downto 0);
signal       cmd_busy10 : std_logic;

signal    out_cmd_val11 : std_logic;
signal        out_cmd11 : std_logic_vector(63 downto 0);
signal       cmd_busy11 : std_logic;

signal sclk_prebuf      : std_logic;
signal serial_clk       : std_logic;
signal sclk_ext         : std_logic;

signal    init_ena_ad9517   : std_logic;
signal   init_done_ad9517   : std_logic;
signal   init_ena_ads5400_0 : std_logic;
signal  init_done_ads5400_0 : std_logic;
signal   init_ena_ads5400_1 : std_logic;
signal  init_done_ads5400_1 : std_logic;
signal  init_ena_dac5681z_0 : std_logic;
signal init_done_dac5681z_0 : std_logic;
signal  init_ena_dac5681z_1 : std_logic;
signal init_done_dac5681z_1 : std_logic;
signal      init_ena_cpld   : std_logic;
signal     init_done_cpld   : std_logic;

signal        spi_n_oe0   : std_logic_vector(5 downto 0);
signal        spi_n_cs0   : std_logic_vector(5 downto 0);
signal        spi_sclk0   : std_logic_vector(5 downto 0);
signal         spi_sdo0   : std_logic_vector(5 downto 0);
signal         spi_sdi0   : std_logic_vector(5 downto 0);

signal        spi_n_oe    : std_logic;
signal        spi_n_cs    : std_logic;
signal        spi_sclk    : std_logic;
signal         spi_sdo    : std_logic;
signal         spi_sdi    : std_logic;

signal adc0_phy_out_clk   : std_logic;
signal adc0_phy_out_data  : std_logic_vector(127 downto 0);

signal adc0_pack_en       : std_logic;
signal adc0_pack_stop     : std_logic;
signal adc0_pack_dval     : std_logic;
signal adc0_pack_data     : std_logic_vector(63 downto 0);

signal adc1_phy_out_clk   : std_logic;
signal adc1_phy_out_data  : std_logic_vector(127 downto 0);

signal adc1_pack_en       : std_logic;
signal adc1_pack_stop     : std_logic;
signal adc1_pack_dval     : std_logic;
signal adc1_pack_data     : std_logic_vector(63 downto 0);

signal dac0_phy_in_clk    : std_logic;
signal dac0_phy_in_dval   : std_logic;
signal dac0_phy_in_data   : std_logic_vector(127 downto 0);

signal dac1_phy_in_clk    : std_logic;
signal dac1_phy_in_dval   : std_logic;
signal dac1_phy_in_data   : std_logic_vector(127 downto 0);

signal fifo0_wr_en        : std_logic;
signal fifo0_usedw        : std_logic_vector(31 downto 0);
signal fifo0_empty        : std_logic;
signal fifo0_full         : std_logic;

signal fifo1_wr_en        : std_logic;
signal fifo1_usedw        : std_logic_vector(31 downto 0);
signal fifo1_empty        : std_logic;
signal fifo1_full         : std_logic;

signal wfm0_load_size     : std_logic_vector(31 downto 0);
signal wfm0_load_start    : std_logic;
signal wfm0_offload_size  : std_logic_vector(31 downto 0);
signal wfm0_offload_start : std_logic;
signal wfm0_offload_stop  : std_logic;

signal wfm1_load_size     : std_logic_vector(31 downto 0);
signal wfm1_load_start    : std_logic;
signal wfm1_offload_size  : std_logic_vector(31 downto 0);
signal wfm1_offload_start : std_logic;
signal wfm1_offload_stop  : std_logic;

signal test_clocks        : std_logic_vector(15 downto 0);

signal ext_trigger_buf    : std_logic;

signal clk_to_fpga_buf    : std_logic;
signal clk_to_fpga        : std_logic;

signal dac0_sync          : std_logic;
signal dac1_sync          : std_logic;
signal dac_sync           : std_logic;

----------------------------------------------------------------------------------------------------
--Component declaration
----------------------------------------------------------------------------------------------------
component fmc110_ctrl is
  generic
  (
    START_ADDR             : std_logic_vector(27 downto 0) := x"0000000";
    STOP_ADDR              : std_logic_vector(27 downto 0) := x"00000FF"
  );
  port (
    rst                    : in  std_logic;

    -- Command Interface
    clk_cmd                : in    std_logic;
    in_cmd_val             : in    std_logic;
    in_cmd                 : in    std_logic_vector(63 downto 0);
    out_cmd_val            : out   std_logic;
    out_cmd                : out   std_logic_vector(63 downto 0);
    cmd_busy               : out   std_logic;

    --External trigger
    ext_trigger_p          : in  std_logic;
    ext_trigger_n          : in  std_logic;
    ext_trigger_buf        : out std_logic;

    --ADC0 FIFO Control
    adc0_clk               : in  std_logic;
    adc0_pack_en           : out std_logic;
    fifo0_wr_en            : out std_logic;
    fifo0_usedw            : in  std_logic_vector(31 downto 0);
    fifo0_empty            : in  std_logic;
    fifo0_full             : in  std_logic;

    --ADC1 FIFO Control
    adc1_clk               : in  std_logic;
    adc1_pack_en           : out std_logic;
    fifo1_wr_en            : out std_logic;
    fifo1_usedw            : in  std_logic_vector(31 downto 0);
    fifo1_empty            : in  std_logic;
    fifo1_full             : in  std_logic;

    --DAC0 WFM Control
    dac0_clk               : in  std_logic;
    wfm0_load_size         : out std_logic_vector(31 downto 0);
    wfm0_load_start        : out std_logic;
    wfm0_offload_size      : out std_logic_vector(31 downto 0);
    wfm0_offload_start     : out std_logic;
    wfm0_offload_stop      : out std_logic;

    --DAC1 WFM Control
    dac1_clk               : in  std_logic;
    wfm1_load_size         : out std_logic_vector(31 downto 0);
    wfm1_load_start        : out std_logic;
    wfm1_offload_size      : out std_logic_vector(31 downto 0);
    wfm1_offload_start     : out std_logic;
    wfm1_offload_stop      : out std_logic;

    pg_m2c                 : in  std_logic;
    prsnt_m2c_l            : in  std_logic

  );
end component;

component ads5400_phy_dp is
  generic
  (
    START_ADDR   : std_logic_vector(27 downto 0) := x"0000000";
    STOP_ADDR    : std_logic_vector(27 downto 0) := x"00000FF"
  );
  port (
    -- Global signals
    rst          : in  std_logic;

    -- Command Interface
    clk_cmd      : in  std_logic;
    in_cmd_val   : in  std_logic;
    in_cmd       : in  std_logic_vector(63 downto 0);
    out_cmd_val  : out std_logic;
    out_cmd      : out std_logic_vector(63 downto 0);
    in_cmd_busy  : out std_logic;

    -- DDR LVDS Interface
    clk_a_p      : in  std_logic;
    clk_a_n      : in  std_logic;

    cha_p        : in  std_logic_vector(11 downto 0);
    cha_n        : in  std_logic_vector(11 downto 0);
    ovra_p       : in  std_logic;
    ovra_n       : in  std_logic;

    chb_p        : in  std_logic_vector(11 downto 0);
    chb_n        : in  std_logic_vector(11 downto 0);
    ovrb_p       : in  std_logic;
    ovrb_n       : in  std_logic;

    -- Output data
    phy_out_clk  : out std_logic;                      -- clock 1/8 of the sample frequecy
    phy_out_data : out std_logic_vector(127 downto 0)  -- 8 samples in parallel in 16-bit format

  );
end component;

component ads5400_phy_sp is
  generic
  (
    START_ADDR   : std_logic_vector(27 downto 0) := x"0000000";
    STOP_ADDR    : std_logic_vector(27 downto 0) := x"00000FF"
  );
  port (
    -- Global signals
    rst          : in  std_logic;

    -- Command Interface
    clk_cmd      : in  std_logic;
    in_cmd_val   : in  std_logic;
    in_cmd       : in  std_logic_vector(63 downto 0);
    out_cmd_val  : out std_logic;
    out_cmd      : out std_logic_vector(63 downto 0);
    in_cmd_busy  : out std_logic;

    -- DDR LVDS Interface
    clk_a_p      : in  std_logic;
    clk_a_n      : in  std_logic;

    cha_p        : in  std_logic_vector(11 downto 0);
    cha_n        : in  std_logic_vector(11 downto 0);
    ovra_p       : in  std_logic;
    ovra_n       : in  std_logic;

    -- Output data
    phy_out_clk  : out std_logic;                      -- clock 1/8 of the sample frequecy
    phy_out_data : out std_logic_vector(127 downto 0)  -- 8 samples in parallel in 16-bit format

  );
end component;

component ads5400_fifo is
  port (
    rst        : in  std_logic;
    -- Input port
    phy_clk    : in  std_logic;
    phy_data   : in  std_logic_vector(127 downto 0);

    fifo_wr_en : in  std_logic;
    fifo_usedw : out std_logic_vector(31 downto 0);
    fifo_empty : out std_logic;
    fifo_full  : out std_logic;

    -- Output port
    if_clk     : in  std_logic;
    if_stop    : in  std_logic;
    if_dval    : out std_logic;
    if_data    : out std_logic_vector(63 downto 0)
  );
end component;

component pack_16to12 is
port (
  clk           : in  std_logic;
  rst           : in  std_logic;

  enable        : in  std_logic;

  data_in_dval  : in  std_logic;
  data_in       : in  std_logic_vector(63 downto 0);
  data_in_stop  : out std_logic;

  data_out_dval : out std_logic;
  data_out      : out std_logic_vector(63 downto 0);
  data_out_stop : in std_logic
);
end component;

component dac5681z_phy is
  generic
  (
    START_ADDR   : std_logic_vector(27 downto 0) := x"0000000";
    STOP_ADDR    : std_logic_vector(27 downto 0) := x"00000FF"
  );
  port (

    -- Global signals
    rst          : in  std_logic;

    -- Command Interface
    clk_cmd      : in  std_logic;
    in_cmd_val   : in  std_logic;
    in_cmd       : in  std_logic_vector(63 downto 0);
    out_cmd_val  : out std_logic;
    out_cmd      : out std_logic_vector(63 downto 0);
    in_cmd_busy  : out std_logic;

    -- Input data
    phy_in_clk   : out std_logic;                      -- clock 1/8 of the sample frequecy
    phy_in_dval  : in  std_logic;                      -- valid signal
    phy_in_data  : in  std_logic_vector(127 downto 0); -- 8 samples in parallel in 16-bit format

    -- DAC interface
    clk_to_fpga  : in  std_logic;

    dac_dclk_p   : out std_logic;
    dac_dclk_n   : out std_logic;
    dac_data_p   : out std_logic_vector(15 downto 0);
    dac_data_n   : out std_logic_vector(15 downto 0);
    dac_sync     : out std_logic

  );
end component;

component dac5681z_wfm  is
port (
   -- Reset port
   rst               : in std_logic;

   -- Load port
   wr_clk            : in std_logic;
   wfm_load_size     : in  std_logic_vector(31 downto 0);
   wfm_load_start    : in  std_logic;

   wfm_in_data       : in std_logic_vector(63 downto 0);
   wfm_in_stop       : out std_logic;
   wfm_in_dval       : in std_logic;

   -- Offload port
   rd_clk            : in std_logic;
   wfm_offload_size  : in  std_logic_vector(31 downto 0);
   wfm_offload_start : in  std_logic;
   wfm_offload_stop  : in  std_logic;

   wfm_out_data      : out std_logic_vector(127 downto 0);
   wfm_out_stop      : in std_logic;
   wfm_out_dval      : out std_logic

);
end component;

component fmc110_cpld_ctrl is
generic (
  START_ADDR      : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR       : std_logic_vector(27 downto 0) := x"00000FF";
  PRESEL          : std_logic_vector(7 downto 0) := x"00"
);
port (
  rst             : in  std_logic;
  clk             : in  std_logic;
  -- Sequence interface
  init_ena        : in  std_logic;
  init_done       : out std_logic;
  -- Command Interface
  clk_cmd         : in  std_logic;
  in_cmd_val      : in  std_logic;
  in_cmd          : in  std_logic_vector(63 downto 0);
  out_cmd_val     : out std_logic;
  out_cmd         : out std_logic_vector(63 downto 0);
  in_cmd_busy     : out std_logic;
  -- SPI control
  spi_n_oe        : out std_logic;
  spi_n_cs        : out std_logic;
  spi_sclk        : out std_logic;
  spi_sdo         : out std_logic;
  spi_sdi         : in  std_logic
);
end component;

component fmc110_ad9517_ctrl is
generic (
  START_ADDR      : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR       : std_logic_vector(27 downto 0) := x"00000FF";
  PRESEL          : std_logic_vector(7 downto 0) := x"00"
);
port (
  rst             : in  std_logic;
  clk             : in  std_logic;
  serial_clk      : in  std_logic;
  sclk_ext        : in  std_logic;
  -- Sequence interface
  init_ena        : in  std_logic;
  init_done       : out std_logic;
  -- Command Interface
  clk_cmd         : in  std_logic;
  in_cmd_val      : in  std_logic;
  in_cmd          : in  std_logic_vector(63 downto 0);
  out_cmd_val     : out std_logic;
  out_cmd         : out std_logic_vector(63 downto 0);
  in_cmd_busy     : out std_logic;
  -- SPI control
  spi_n_oe        : out std_logic;
  spi_n_cs        : out std_logic;
  spi_sclk        : out std_logic;
  spi_sdo         : out std_logic;
  spi_sdi         : in  std_logic
);
end component;

component fmc110_ads5400_ctrl is
generic (
  START_ADDR      : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR       : std_logic_vector(27 downto 0) := x"00000FF";
  PRESEL          : std_logic_vector(7 downto 0) := x"00"
);
port (
  rst             : in  std_logic;
  clk             : in  std_logic;
  serial_clk      : in  std_logic;
  sclk_ext        : in  std_logic;
  -- Sequence interface
  init_ena        : in  std_logic;
  init_done       : out std_logic;
  -- Command Interface
  clk_cmd         : in  std_logic;
  in_cmd_val      : in  std_logic;
  in_cmd          : in  std_logic_vector(63 downto 0);
  out_cmd_val     : out std_logic;
  out_cmd         : out std_logic_vector(63 downto 0);
  in_cmd_busy     : out std_logic;
  -- SPI control
  spi_n_oe        : out std_logic;
  spi_n_cs        : out std_logic;
  spi_sclk        : out std_logic;
  spi_sdo         : out std_logic;
  spi_sdi         : in  std_logic
);
end component;

component fmc110_dac5681z_ctrl is
generic (
  START_ADDR      : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR       : std_logic_vector(27 downto 0) := x"00000FF";
  PRESEL          : std_logic_vector(7 downto 0) := x"00"
);
port (
  rst             : in  std_logic;
  clk             : in  std_logic;
  serial_clk      : in  std_logic;
  sclk_ext        : in  std_logic;
  -- Sequence interface
  init_ena        : in  std_logic;
  init_done       : out std_logic;
  -- Command Interface
  clk_cmd         : in  std_logic;
  in_cmd_val      : in  std_logic;
  in_cmd          : in  std_logic_vector(63 downto 0);
  out_cmd_val     : out std_logic;
  out_cmd         : out std_logic_vector(63 downto 0);
  in_cmd_busy     : out std_logic;
  -- SPI control
  spi_n_oe        : out std_logic;
  spi_n_cs        : out std_logic;
  spi_sclk        : out std_logic;
  spi_sdo         : out std_logic;
  spi_sdi         : in  std_logic
);
end component;

component sip_freq_cnt16 is
generic (
  START_ADDR      : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR       : std_logic_vector(27 downto 0) := x"0000001"
);
port (
  -- Command Interface
  clk_cmd         : in  std_logic;
  in_cmd_val      : in  std_logic;
  in_cmd          : in  std_logic_vector(63 downto 0);
  out_cmd_val     : out std_logic;
  out_cmd         : out std_logic_vector(63 downto 0);
  -- Clocks Interface
  reset           : in  std_logic;
  reference_clock : in  std_logic;
  test_clocks     : in  std_logic_vector(15 downto 0)
);
end component;

begin

----------------------------------------------------------------------------------------------------
-- Trigger control
----------------------------------------------------------------------------------------------------
fmc110_ctrl_inst : fmc110_ctrl
  generic map (
    START_ADDR         => START_ADDR_FMC110_CTRL,
    STOP_ADDR          => STOP_ADDR_FMC110_CTRL
  )
  port map (
    rst                => rst,

    clk_cmd            => clk_cmd,
    in_cmd_val         => in_cmd_val,
    in_cmd             => in_cmd,
    out_cmd_val        => out_cmd_val0,
    out_cmd            => out_cmd0,
    cmd_busy           => cmd_busy0,

    ext_trigger_p      => ext_trigger_p,
    ext_trigger_n      => ext_trigger_n,
    ext_trigger_buf    => ext_trigger_buf,

    adc0_clk           => adc0_phy_out_clk,
    adc0_pack_en       => adc0_pack_en,
    fifo0_wr_en        => fifo0_wr_en,
    fifo0_usedw        => fifo0_usedw,
    fifo0_empty        => fifo0_empty,
    fifo0_full         => fifo0_full,

    adc1_clk           => adc1_phy_out_clk,
    adc1_pack_en       => adc1_pack_en,
    fifo1_wr_en        => fifo1_wr_en,
    fifo1_usedw        => fifo1_usedw,
    fifo1_empty        => fifo1_empty,
    fifo1_full         => fifo1_full,

    dac0_clk           => dac0_phy_in_clk,
    wfm0_load_size     => wfm0_load_size,
    wfm0_load_start    => wfm0_load_start,
    wfm0_offload_size  => wfm0_offload_size,
    wfm0_offload_start => wfm0_offload_start,
    wfm0_offload_stop  => wfm0_offload_stop,

    dac1_clk           => dac1_phy_in_clk,
    wfm1_load_size     => wfm1_load_size,
    wfm1_load_start    => wfm1_load_start,
    wfm1_offload_size  => wfm1_offload_size,
    wfm1_offload_start => wfm1_offload_start,
    wfm1_offload_stop  => wfm1_offload_stop,

    pg_m2c             => pg_m2c,
    prsnt_m2c_l        => prsnt_m2c_l

  );

----------------------------------------------------------------------------------------------------
-- ADS5400 Physical Data Interface for ADC0
----------------------------------------------------------------------------------------------------
ads5400_phy_inst0 : ads5400_phy_sp
  generic map
  (
    START_ADDR   => START_ADDR_ADS5400_PHY0,
    STOP_ADDR    => STOP_ADDR_ADS5400_PHY0
  )
  port map (
    rst          => rst,

    clk_cmd      => clk_cmd,
    in_cmd_val   => in_cmd_val,
    in_cmd       => in_cmd,
    out_cmd_val  => out_cmd_val1,
    out_cmd      => out_cmd1,
    in_cmd_busy  => cmd_busy1,

    clk_a_p      => adc0_clka_p,
    clk_a_n      => adc0_clka_n,

    cha_p        => adc0_da_p,
    cha_n        => adc0_da_n,
    ovra_p       => adc0_ovra_p,
    ovra_n       => adc0_ovra_n,

    --chb_p        => adc0_db_p,
    --chb_n        => adc0_db_n,
    --ovrb_p       => adc0_ovrb_p,
    --ovrb_n       => adc0_ovrb_n,

    phy_out_clk  => adc0_phy_out_clk,
    phy_out_data => adc0_phy_out_data

  );

----------------------------------------------------------------------------------------------------
-- On chip sample data storage for ADC0
----------------------------------------------------------------------------------------------------
ads5400_fifo_inst0 : ads5400_fifo
port map (
  rst        => rst,
  phy_clk    => adc0_phy_out_clk,
  phy_data   => adc0_phy_out_data,
  fifo_wr_en => fifo0_wr_en,
  fifo_usedw => fifo0_usedw,
  fifo_empty => fifo0_empty,
  fifo_full  => fifo0_full,
  if_clk     => clk_cmd,
  if_stop    => adc0_pack_stop,
  if_dval    => adc0_pack_dval,
  if_data    => adc0_pack_data
);

----------------------------------------------------------------------------------------------------
-- Data Packer for for ADC0
----------------------------------------------------------------------------------------------------
pack_16to12_inst0 : pack_16to12
port map (
  clk           => clk_cmd,
  rst           => rst,
  enable        => adc0_pack_en,
  data_in_stop  => adc0_pack_stop,
  data_in_dval  => adc0_pack_dval,
  data_in       => adc0_pack_data,
  data_out_stop => adc0_out_stop,
  data_out_dval => adc0_out_dval,
  data_out      => adc0_out_data
);

----------------------------------------------------------------------------------------------------
-- ADS5400 Physical Data Interface for ADC1
----------------------------------------------------------------------------------------------------
ads5400_phy_inst1 : ads5400_phy_sp
  generic map
  (
    START_ADDR   => START_ADDR_ADS5400_PHY1,
    STOP_ADDR    => STOP_ADDR_ADS5400_PHY1
  )
  port map (
    rst          => rst,

    clk_cmd      => clk_cmd,
    in_cmd_val   => in_cmd_val,
    in_cmd       => in_cmd,
    out_cmd_val  => out_cmd_val2,
    out_cmd      => out_cmd2,
    in_cmd_busy  => cmd_busy2,

    clk_a_p      => adc1_clka_p,
    clk_a_n      => adc1_clka_n,

    cha_p        => adc1_da_p,
    cha_n        => adc1_da_n,
    ovra_p       => adc1_ovra_p,
    ovra_n       => adc1_ovra_n,

    phy_out_clk  => adc1_phy_out_clk,
    phy_out_data => adc1_phy_out_data

  );

----------------------------------------------------------------------------------------------------
-- On chip sample data storage for ADC1
----------------------------------------------------------------------------------------------------
ads5400_fifo_inst1 : ads5400_fifo
port map (
  rst        => rst,
  phy_clk    => adc1_phy_out_clk,
  phy_data   => adc1_phy_out_data,
  fifo_wr_en => fifo1_wr_en,
  fifo_usedw => fifo1_usedw,
  fifo_empty => fifo1_empty,
  fifo_full  => fifo1_full,
  if_clk     => clk_cmd,
  if_stop    => adc1_pack_stop,
  if_dval    => adc1_pack_dval,
  if_data    => adc1_pack_data
);

----------------------------------------------------------------------------------------------------
-- Data Packer for for ADC1
----------------------------------------------------------------------------------------------------
pack_16to12_inst1 : pack_16to12
port map (
  clk           => clk_cmd,
  rst           => rst,
  enable        => adc1_pack_en,
  data_in_stop  => adc1_pack_stop,
  data_in_dval  => adc1_pack_dval,
  data_in       => adc1_pack_data,
  data_out_stop => adc1_out_stop,
  data_out_dval => adc1_out_dval,
  data_out      => adc1_out_data
);

----------------------------------------------------------------------------------------------------
-- DAC5681Z Physical Data Interface for DAC0
----------------------------------------------------------------------------------------------------
dac5681z_phy_inst0 : dac5681z_phy
  generic map
  (
    START_ADDR   => START_ADDR_DAC5681Z_PHY0,
    STOP_ADDR    => STOP_ADDR_DAC5681Z_PHY0
  )
  port map (
    rst          => rst,

    clk_cmd      => clk_cmd,
    in_cmd_val   => in_cmd_val,
    in_cmd       => in_cmd,
    out_cmd_val  => out_cmd_val3,
    out_cmd      => out_cmd3,
    in_cmd_busy  => cmd_busy3,

    phy_in_clk   => dac0_phy_in_clk,
    phy_in_dval  => dac0_phy_in_dval,
    phy_in_data  => dac0_phy_in_data,

    clk_to_fpga  => clk_to_fpga,

    dac_dclk_p   => dac0_dclk_p,
    dac_dclk_n   => dac0_dclk_n,
    dac_data_p   => dac0_data_p,
    dac_data_n   => dac0_data_n,
    dac_sync     => dac0_sync

  );

----------------------------------------------------------------------------------------------------
-- DAC5681Z Wave Form Memory for DAC0
----------------------------------------------------------------------------------------------------
dac5681z_wfm_inst0 : dac5681z_wfm
port map (
   rst               => rst,

   wr_clk            => clk_cmd,
   wfm_load_size     => wfm0_load_size,
   wfm_load_start    => wfm0_load_start,

   wfm_in_data       => dac0_in_data,
   wfm_in_stop       => dac0_in_stop,
   wfm_in_dval       => dac0_in_dval,

   rd_clk            => dac0_phy_in_clk,
   wfm_offload_size  => wfm0_offload_size,
   wfm_offload_start => wfm0_offload_start,
   wfm_offload_stop  => wfm0_offload_stop,

   wfm_out_data      => dac0_phy_in_data,
   wfm_out_stop      => '0',
   wfm_out_dval      => dac0_phy_in_dval
);

----------------------------------------------------------------------------------------------------
-- DAC5681Z Physical Data Interface for DAC1
----------------------------------------------------------------------------------------------------
dac5681z_phy_inst1 : dac5681z_phy
  generic map
  (
    START_ADDR   => START_ADDR_DAC5681Z_PHY1,
    STOP_ADDR    => STOP_ADDR_DAC5681Z_PHY1
  )
  port map (
    rst          => rst,

    clk_cmd      => clk_cmd,
    in_cmd_val   => in_cmd_val,
    in_cmd       => in_cmd,
    out_cmd_val  => out_cmd_val4,
    out_cmd      => out_cmd4,
    in_cmd_busy  => cmd_busy4,

    phy_in_clk   => dac1_phy_in_clk,
    phy_in_dval  => dac1_phy_in_dval,
    phy_in_data  => dac1_phy_in_data,

    clk_to_fpga  => clk_to_fpga,

    dac_dclk_p   => dac1_dclk_p,
    dac_dclk_n   => dac1_dclk_n,
    dac_data_p   => dac1_data_p,
    dac_data_n   => dac1_data_n,
    dac_sync     => dac1_sync

  );

----------------------------------------------------------------------------------------------------
-- DAC5681Z Wave Form Memory for DAC1
----------------------------------------------------------------------------------------------------
dac5681z_wfm_inst1 : dac5681z_wfm
port map (
   rst               => rst,

   wr_clk            => clk_cmd,
   wfm_load_size     => wfm1_load_size,
   wfm_load_start    => wfm1_load_start,

   wfm_in_data       => dac1_in_data,
   wfm_in_stop       => dac1_in_stop,
   wfm_in_dval       => dac1_in_dval,

   rd_clk            => dac1_phy_in_clk,
   wfm_offload_size  => wfm1_offload_size,
   wfm_offload_start => wfm1_offload_start,
   wfm_offload_stop  => wfm1_offload_stop,

   wfm_out_data      => dac1_phy_in_data,
   wfm_out_stop      => '0',
   wfm_out_dval      => dac1_phy_in_dval
);

----------------------------------------------------------------------------------------------------
-- Generate serial clocks for SPI (max 6.66MHz, due to Tddata of 75ns on ADS5400)
----------------------------------------------------------------------------------------------------

process (clk)
  -- Divide by 2^5 = 32, CLKmax = 32 x 6.66MHz
  variable clk_div : std_logic_vector(4 downto 0) := (others => '0');
begin
  if (rising_edge(clk)) then
    clk_div    := clk_div + '1';
    -- The slave samples the data on the rising edge of SCLK.
    -- therefore we make sure the external clock is slightly
    -- after the internal clock.
    sclk_ext <= clk_div(clk_div'length-1);
    sclk_prebuf <= sclk_ext;
  end if;
end process;

bufg_sclk : bufg
port map (
  i => sclk_prebuf,
  o => serial_clk
);

----------------------------------------------------------------------------------------------------
-- SPI Interface controlling the clock IC
----------------------------------------------------------------------------------------------------
ad9517_ctrl_inst0 : fmc110_ad9517_ctrl
generic map (
  START_ADDR      => START_ADDR_AD9517_CTRL,
  STOP_ADDR       => STOP_ADDR_AD9517_CTRL,
  PRESEL          => PRESEL_CLK0
)
port map (
  rst             => rst,
  clk             => clk,
  serial_clk      => serial_clk,
  sclk_ext        => sclk_ext,

  init_ena        => init_ena_ad9517,
  init_done       => init_done_ad9517,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val5,
  out_cmd         => out_cmd5,
  in_cmd_busy     => cmd_busy5,

  spi_n_oe        => spi_n_oe0(0),
  spi_n_cs        => spi_n_cs0(0),
  spi_sclk        => spi_sclk0(0),
  spi_sdo         => spi_sdo0(0),
  spi_sdi         => spi_sdi0(0)
);

----------------------------------------------------------------------------------------------------
-- SPI interface controlling ADC chip 0
----------------------------------------------------------------------------------------------------
ads5400_ctrl_inst0 : fmc110_ads5400_ctrl
generic map (
  START_ADDR      => START_ADDR_ADS5400_CTRL0,
  STOP_ADDR       => STOP_ADDR_ADS5400_CTRL0,
  PRESEL          => PRESEL_ADC0
)
port map (
  rst             => rst,
  clk             => clk,
  serial_clk      => serial_clk,
  sclk_ext        => sclk_ext,

  init_ena        => init_ena_ads5400_0,
  init_done       => init_done_ads5400_0,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val6,
  out_cmd         => out_cmd6,
  in_cmd_busy     => cmd_busy6,

  spi_n_oe        => spi_n_oe0(1),
  spi_n_cs        => spi_n_cs0(1),
  spi_sclk        => spi_sclk0(1),
  spi_sdo         => spi_sdo0(1),
  spi_sdi         => spi_sdi0(1)
);

----------------------------------------------------------------------------------------------------
-- SPI interface controlling ADC chip 0
----------------------------------------------------------------------------------------------------
ads5400_ctrl_inst1 : fmc110_ads5400_ctrl
generic map (
  START_ADDR      => START_ADDR_ADS5400_CTRL1,
  STOP_ADDR       => STOP_ADDR_ADS5400_CTRL1,
  PRESEL          => PRESEL_ADC1
)
port map (
  rst             => rst,
  clk             => clk,
  serial_clk      => serial_clk,
  sclk_ext        => sclk_ext,

  init_ena        => init_ena_ads5400_1,
  init_done       => init_done_ads5400_1,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val7,
  out_cmd         => out_cmd7,
  in_cmd_busy     => cmd_busy7,

  spi_n_oe        => spi_n_oe0(2),
  spi_n_cs        => spi_n_cs0(2),
  spi_sclk        => spi_sclk0(2),
  spi_sdo         => spi_sdo0(2),
  spi_sdi         => spi_sdi0(2)
);

----------------------------------------------------------------------------------------------------
-- SPI interface controlling DAC chip 0
----------------------------------------------------------------------------------------------------
dac5681z_ctrl_inst0 : fmc110_dac5681z_ctrl
generic map (
  START_ADDR      => START_ADDR_DAC5681Z_CTRL0,
  STOP_ADDR       => STOP_ADDR_DAC5681Z_CTRL0,
  PRESEL          => PRESEL_DAC0
)
port map (
  rst             => rst,
  clk             => clk,
  serial_clk      => serial_clk,
  sclk_ext        => sclk_ext,

  init_ena        => init_ena_dac5681z_0,
  init_done       => init_done_dac5681z_0,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val8,
  out_cmd         => out_cmd8,
  in_cmd_busy     => cmd_busy8,

  spi_n_oe        => spi_n_oe0(3),
  spi_n_cs        => spi_n_cs0(3),
  spi_sclk        => spi_sclk0(3),
  spi_sdo         => spi_sdo0(3),
  spi_sdi         => spi_sdi0(3)
);

----------------------------------------------------------------------------------------------------
-- SPI interface controlling DAC chip 1
----------------------------------------------------------------------------------------------------
dac5681z_ctrl_inst1 : fmc110_dac5681z_ctrl
generic map (
  START_ADDR      => START_ADDR_DAC5681Z_CTRL1,
  STOP_ADDR       => STOP_ADDR_DAC5681Z_CTRL1,
  PRESEL          => PRESEL_DAC1
)
port map (
  rst             => rst,
  clk             => clk,
  serial_clk      => serial_clk,
  sclk_ext        => sclk_ext,

  init_ena        => init_ena_dac5681z_1,
  init_done       => init_done_dac5681z_1,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val9,
  out_cmd         => out_cmd9,
  in_cmd_busy     => cmd_busy9,

  spi_n_oe        => spi_n_oe0(4),
  spi_n_cs        => spi_n_cs0(4),
  spi_sclk        => spi_sclk0(4),
  spi_sdo         => spi_sdo0(4),
  spi_sdi         => spi_sdi0(4)
);

----------------------------------------------------------------------------------------------------
-- SPI Interface accessing internal CPLD registers
----------------------------------------------------------------------------------------------------
fmc110_cpld_ctrl_inst : fmc110_cpld_ctrl
generic map (
  START_ADDR      => START_ADDR_CPLD_CTRL,
  STOP_ADDR       => STOP_ADDR_CPLD_CTRL,
  PRESEL          => PRESEL_CPLD
)
port map (
  rst             => rst,
  clk             => clk,

  init_ena        => init_ena_cpld,
  init_done       => init_done_cpld,

  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val10,
  out_cmd         => out_cmd10,
  in_cmd_busy     => cmd_busy10,

  spi_n_oe        => spi_n_oe0(5),
  spi_n_cs        => spi_n_cs0(5),
  spi_sclk        => spi_sclk0(5),
  spi_sdo         => spi_sdo0(5),
  spi_sdi         => spi_sdi0(5)
);

----------------------------------------------------------------------------------------------------
-- Sequence SPI initialization
----------------------------------------------------------------------------------------------------

-- start ad9517 init after power up
init_ena_ad9517 <= '1';

-- start ads5400 init after AD9517 init done
init_ena_ads5400_0 <= init_done_ad9517;
init_ena_ads5400_1 <= init_done_ads5400_0;

-- start dac5681z init after ads5400 init done
init_ena_dac5681z_0 <= init_done_ads5400_1;
init_ena_dac5681z_1 <= init_done_dac5681z_0;

-- start cpld init after dac5681z init done
init_ena_cpld <= '0'; -- no init required

----------------------------------------------------------------------------------------------------
-- SPI PHY, shared SPI bus
----------------------------------------------------------------------------------------------------
spi_sclk <= spi_sclk0(0) when spi_n_cs0(0) = '0' else
            spi_sclk0(1) when spi_n_cs0(1) = '0' else
            spi_sclk0(2) when spi_n_cs0(2) = '0' else
            spi_sclk0(3) when spi_n_cs0(3) = '0' else
            spi_sclk0(4) when spi_n_cs0(4) = '0' else
            spi_sclk0(5) when spi_n_cs0(5) = '0' else '0';

iobuf_cpld0 : iobuf
port map (
  I  => spi_sclk,
  O  => open,
  IO => fmc_to_cpld(0),
  T  => '0'
);

spi_n_oe <= and_reduce(spi_n_oe0);
spi_n_cs <= and_reduce(spi_n_cs0);

iobuf_cpld1 : iobuf
port map (
  I  => spi_n_cs,
  O  => open,
  IO => fmc_to_cpld(1),
  T  => '0'
);

spi_sdo <= spi_sdo0(0) when spi_n_oe0(0) = '0' else
           spi_sdo0(1) when spi_n_oe0(1) = '0' else
           spi_sdo0(2) when spi_n_oe0(2) = '0' else
           spi_sdo0(3) when spi_n_oe0(3) = '0' else
           spi_sdo0(4) when spi_n_oe0(4) = '0' else
           spi_sdo0(5) when spi_n_oe0(5) = '0' else '1';

iobuf_cpld2 : iobuf
port map (
  I  => spi_sdo,
  O  => spi_sdi,
  IO => fmc_to_cpld(2),
  T  => spi_n_oe
);

spi_sdi0(0) <= spi_sdi;
spi_sdi0(1) <= spi_sdi;
spi_sdi0(2) <= spi_sdi;
spi_sdi0(3) <= spi_sdi;
spi_sdi0(4) <= spi_sdi;
spi_sdi0(5) <= spi_sdi;

iobuf_cpld3 : iobuf
port map (
  I  => '0',
  O  => open,
  IO => fmc_to_cpld(3),
  T  => '1'
);

----------------------------------------------------------------------------------------------------
-- Frequency counter
----------------------------------------------------------------------------------------------------
sip_freq_cnt16_inst : sip_freq_cnt16
generic map (
  START_ADDR      => START_ADDR_FREQ_CNT,
  STOP_ADDR       => STOP_ADDR_FREQ_CNT
)
port map (
  clk_cmd         => clk_cmd,
  in_cmd_val      => in_cmd_val,
  in_cmd          => in_cmd,
  out_cmd_val     => out_cmd_val11,
  out_cmd         => out_cmd11,
  reset           => rst,
  reference_clock => clk_cmd,
  test_clocks     => test_clocks
);
cmd_busy11 <= '0';

test_clocks(0) <= clk_cmd;
test_clocks(1) <= adc0_phy_out_clk;
test_clocks(2) <= dac0_phy_in_clk;
test_clocks(3) <= ext_trigger_buf;
test_clocks(4) <= adc1_phy_out_clk;
test_clocks(5) <= dac1_phy_in_clk;
test_clocks(6) <= clk_to_fpga;
test_clocks(15 downto 7) <= (others => '0');

----------------------------------------------------------------------------------------------------
-- Command out merge & pipeline
----------------------------------------------------------------------------------------------------
process (rst, clk_cmd)
begin
  if (rst = '1') then

    out_cmd_val  <= '0';
    out_cmd      <= (others => '0');
    cmd_busy     <= '0';

  elsif (rising_edge(clk_cmd)) then

    out_cmd_val  <= out_cmd_val0  or out_cmd_val1 or out_cmd_val2 or out_cmd_val3 or out_cmd_val4 or
                    out_cmd_val5  or out_cmd_val6 or out_cmd_val7 or out_cmd_val8 or out_cmd_val9 or
                    out_cmd_val10 or out_cmd_val11;
    out_cmd      <=     out_cmd0  or     out_cmd1 or     out_cmd2 or     out_cmd3 or     out_cmd4 or
                        out_cmd5  or     out_cmd6 or     out_cmd7 or     out_cmd8 or     out_cmd9 or
                        out_cmd10 or    out_cmd11;
    cmd_busy     <=    cmd_busy0  or    cmd_busy1 or    cmd_busy2 or    cmd_busy3 or    cmd_busy4 or
                       cmd_busy5  or    cmd_busy6 or    cmd_busy7  or   cmd_busy8 or    cmd_busy9 or
                       cmd_busy10 or   cmd_busy11;

  end if;
end process;

----------------------------------------------------------------------------------------------------
-- DAC clock input
----------------------------------------------------------------------------------------------------
ibufds_ref_clk : ibufds
generic map (
  IOSTANDARD => "LVDS_25",
  DIFF_TERM => TRUE
)
port map (
  i  => clk_to_fpga_p,
  ib => clk_to_fpga_n,
  o  => clk_to_fpga_buf
);

bufg_ref_clk : bufg
port map (
  i => clk_to_fpga_buf,
  o => clk_to_fpga
);

----------------------------------------------------------------------------------------------------
-- Reset output
----------------------------------------------------------------------------------------------------

--adc_sync_out : obufds (Not used on the VC707, signals are mapped to GPIO_LED 6 and 7)
--port map (
--  i  => '0',
--  o  => sync_from_fpga_p,
--  ob => sync_from_fpga_n
--);

adc_sync_out_p : obuf
port map (
  o  => sync_from_fpga_p,
  i  => '0'
);

adc_sync_out_n : obuf
port map (
  o => sync_from_fpga_n,
  i  => '1'
);

----------------------------------------------------------------------------------------------------
-- Sync output
----------------------------------------------------------------------------------------------------
dac_sync <= dac0_sync or dac1_sync;

dac_sync_out : obufds
port map (
  o  => dac_sync_p,
  ob => dac_sync_n,
  i  => dac_sync
);

----------------------------------------------------------------------------------------------------
-- Front IO
----------------------------------------------------------------------------------------------------

front_io: for i in 0 to 3 generate

  front_io_fmc(i) <= '0';

end generate;

----------------------------------------------------------------------------------------------------
-- End
----------------------------------------------------------------------------------------------------
end fmc110_if_syn;
