-------------------------------------------------------------------------------------
-- FILE NAME : sip_fmc110.vhd
--
-- AUTHOR    : StellarIP (c) 4DSP
--
-- COMPANY   : 4DSP
--
-- ITEM      : 1
--
-- UNITS     : Entity       - sip_fmc110
--             architecture - arch_sip_fmc110
--
-- LANGUAGE  : VHDL
--
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
-- DESCRIPTION
-- ===========
--
-- sip_fmc110
-- Notes: sip_fmc110
-------------------------------------------------------------------------------------
--  Disclaimer: LIMITED WARRANTY AND DISCLAIMER. These designs are
--              provided to you as is.  4DSP specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. 4DSP does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, 4DSP does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will 4DSP or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if 4DSP
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
----------------------------------------------
--
-------------------------------------------------------------------------------------
--
-------------------------------------------------------------------------------------
--library declaration
-------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_unsigned.all ;
use ieee.std_logic_misc.all ;

-------------------------------------------------------------------------------------
--Entity Declaration
-------------------------------------------------------------------------------------
entity sip_fmc110  is
generic (
  GLOBAL_START_ADDR_GEN                    : std_logic_vector(27 downto 0);
  GLOBAL_STOP_ADDR_GEN                     : std_logic_vector(27 downto 0);
  PRIVATE_START_ADDR_GEN                   : std_logic_vector(27 downto 0);
  PRIVATE_STOP_ADDR_GEN                    : std_logic_vector(27 downto 0)
);
port (
--Wormhole 'clk' of type 'clkin':
   clk_clkin                               : in    std_logic_vector(31 downto 0);

--Wormhole 'rst' of type 'rst_in':
   rst_rstin                               : in    std_logic_vector(31 downto 0);

--Wormhole 'cmdclk_in' of type 'cmdclk_in':
   cmdclk_in_cmdclk                        : in    std_logic;

--Wormhole 'cmd_in' of type 'cmd_in':
   cmd_in_cmdin                            : in    std_logic_vector(63 downto 0);
   cmd_in_cmdin_val                        : in    std_logic;

--Wormhole 'cmd_out' of type 'cmd_out':
   cmd_out_cmdout                          : out   std_logic_vector(63 downto 0);
   cmd_out_cmdout_val                      : out   std_logic;

--Wormhole 'adc0' of type 'wh_out':
   adc0_out_stop                           : in    std_logic;
   adc0_out_dval                           : out   std_logic;
   adc0_out_data                           : out   std_logic_vector(63 downto 0);

--Wormhole 'adc1' of type 'wh_out':
   adc1_out_stop                           : in    std_logic;
   adc1_out_dval                           : out   std_logic;
   adc1_out_data                           : out   std_logic_vector(63 downto 0);

--Wormhole 'dac0' of type 'wh_in':
   dac0_in_stop                            : out   std_logic;
   dac0_in_dval                            : in    std_logic;
   dac0_in_data                            : in    std_logic_vector(63 downto 0);

--Wormhole 'dac1' of type 'wh_in':
   dac1_in_stop                            : out   std_logic;
   dac1_in_dval                            : in    std_logic;
   dac1_in_data                            : in    std_logic_vector(63 downto 0);

--Wormhole 'ext_fmc110' of type 'ext_fmc110':
   fmc_to_cpld                             : inout std_logic_vector(3 downto 0);
   front_io_fmc                            : inout std_logic_vector(3 downto 0);

   clk_to_fpga_p                           : in    std_logic;
   clk_to_fpga_n                           : in    std_logic;
   ext_trigger_p                           : in    std_logic;
   ext_trigger_n                           : in    std_logic;
   sync_from_fpga_p                        : out   std_logic;
   sync_from_fpga_n                        : out   std_logic;

   adc0_clka_p                             : in    std_logic;
   adc0_clka_n                             : in    std_logic;
   adc0_da_p                               : in    std_logic_vector(11 downto 0);
   adc0_da_n                               : in    std_logic_vector(11 downto 0);
   adc0_ovra_p                             : in    std_logic;
   adc0_ovra_n                             : in    std_logic;
   adc0_db_p                               : in    std_logic_vector(11 downto 0);
   adc0_db_n                               : in    std_logic_vector(11 downto 0);
   adc0_ovrb_p                             : in    std_logic;
   adc0_ovrb_n                             : in    std_logic;

   adc1_clka_p                             : in    std_logic;
   adc1_clka_n                             : in    std_logic;
   adc1_da_p                               : in    std_logic_vector(11 downto 0);
   adc1_da_n                               : in    std_logic_vector(11 downto 0);
   adc1_ovra_p                             : in    std_logic;
   adc1_ovra_n                             : in    std_logic;

   dac_sync_p                              : out   std_logic;
   dac_sync_n                              : out   std_logic;

   dac0_dclk_p                             : out   std_logic;
   dac0_dclk_n                             : out   std_logic;
   dac0_data_p                             : out   std_logic_vector(15 downto 0);
   dac0_data_n                             : out   std_logic_vector(15 downto 0);

   dac1_dclk_p                             : out   std_logic;
   dac1_dclk_n                             : out   std_logic;
   dac1_data_p                             : out   std_logic_vector(15 downto 0);
   dac1_data_n                             : out   std_logic_vector(15 downto 0);

   pg_m2c                                  : in    std_logic;
   prsnt_m2c_l                             : in    std_logic

);
end entity sip_fmc110;

-------------------------------------------------------------------------------------
--Architecture declaration
-------------------------------------------------------------------------------------
architecture arch_sip_fmc110 of sip_fmc110 is

-------------------------------------------------------------------------------------
--Constants declaration
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
--Signal declaration
-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------
--Components declarations
-------------------------------------------------------------------------------------
component fmc110_if is
generic (
  START_ADDR       : std_logic_vector(27 downto 0) := x"0000000";
  STOP_ADDR        : std_logic_vector(27 downto 0) := x"00000FF"
);
port (
  -- Global signals
  rst              : in    std_logic;
  clk              : in    std_logic;

  -- Command Interface
  clk_cmd          : in    std_logic;
  in_cmd_val       : in    std_logic;
  in_cmd           : in    std_logic_vector(63 downto 0);
  out_cmd_val      : out   std_logic;
  out_cmd          : out   std_logic_vector(63 downto 0);
  cmd_busy         : out   std_logic;

  --Output ports for ADC data
  adc0_out_stop    : in    std_logic;
  adc0_out_dval    : out   std_logic;
  adc0_out_data    : out   std_logic_vector(63 downto 0);
  adc1_out_stop    : in    std_logic;
  adc1_out_dval    : out   std_logic;
  adc1_out_data    : out   std_logic_vector(63 downto 0);

  --Input ports for DAC data
  dac0_in_stop     : out   std_logic;
  dac0_in_dval     : in    std_logic;
  dac0_in_data     : in    std_logic_vector(63 downto 0);
  dac1_in_stop     : out   std_logic;
  dac1_in_dval     : in    std_logic;
  dac1_in_data     : in    std_logic_vector(63 downto 0);

  --External signals
  fmc_to_cpld      : inout std_logic_vector(3 downto 0);
  front_io_fmc     : inout std_logic_vector(3 downto 0);

  clk_to_fpga_p    : in    std_logic;
  clk_to_fpga_n    : in    std_logic;
  ext_trigger_p    : in    std_logic;
  ext_trigger_n    : in    std_logic;
  sync_from_fpga_p : out   std_logic;
  sync_from_fpga_n : out   std_logic;

  adc0_clka_p      : in    std_logic;
  adc0_clka_n      : in    std_logic;
  adc0_da_p        : in    std_logic_vector(11 downto 0);
  adc0_da_n        : in    std_logic_vector(11 downto 0);
  adc0_ovra_p      : in    std_logic;
  adc0_ovra_n      : in    std_logic;
  adc0_db_p        : in    std_logic_vector(11 downto 0);
  adc0_db_n        : in    std_logic_vector(11 downto 0);
  adc0_ovrb_p      : in    std_logic;
  adc0_ovrb_n      : in    std_logic;

  adc1_clka_p      : in    std_logic;
  adc1_clka_n      : in    std_logic;
  adc1_da_p        : in    std_logic_vector(11 downto 0);
  adc1_da_n        : in    std_logic_vector(11 downto 0);
  adc1_ovra_p      : in    std_logic;
  adc1_ovra_n      : in    std_logic;

  dac_sync_p       : out   std_logic;
  dac_sync_n       : out   std_logic;

  dac0_dclk_p      : out   std_logic;
  dac0_dclk_n      : out   std_logic;
  dac0_data_p      : out   std_logic_vector(15 downto 0);
  dac0_data_n      : out   std_logic_vector(15 downto 0);

  dac1_dclk_p      : out   std_logic;
  dac1_dclk_n      : out   std_logic;
  dac1_data_p      : out   std_logic_vector(15 downto 0);
  dac1_data_n      : out   std_logic_vector(15 downto 0);

  pg_m2c           : in    std_logic;
  prsnt_m2c_l      : in    std_logic

);
end component;



begin

-------------------------------------------------------------------------------------
--Components instantiations
-------------------------------------------------------------------------------------
fmc110_if_inst : fmc110_if
generic map
(
  START_ADDR      => PRIVATE_START_ADDR_GEN,
  STOP_ADDR       => PRIVATE_STOP_ADDR_GEN
)
port map (
  rst             => rst_rstin(2),
  clk             => cmdclk_in_cmdclk,

  clk_cmd         => cmdclk_in_cmdclk,
  in_cmd_val      => cmd_in_cmdin_val,
  in_cmd          => cmd_in_cmdin,
  out_cmd_val     => cmd_out_cmdout_val,
  out_cmd         => cmd_out_cmdout,
  cmd_busy        => open,

  adc0_out_stop   => adc0_out_stop,
  adc0_out_dval   => adc0_out_dval,
  adc0_out_data   => adc0_out_data,
  adc1_out_stop   => adc1_out_stop,
  adc1_out_dval   => adc1_out_dval,
  adc1_out_data   => adc1_out_data,

  dac0_in_stop    => dac0_in_stop,
  dac0_in_dval    => dac0_in_dval,
  dac0_in_data    => dac0_in_data,
  dac1_in_stop    => dac1_in_stop,
  dac1_in_dval    => dac1_in_dval,
  dac1_in_data    => dac1_in_data,

  fmc_to_cpld     => fmc_to_cpld,
  front_io_fmc    => front_io_fmc,

  clk_to_fpga_p   => clk_to_fpga_p,
  clk_to_fpga_n   => clk_to_fpga_n,
  ext_trigger_p   => ext_trigger_p,
  ext_trigger_n   => ext_trigger_n,
  sync_from_fpga_p=> sync_from_fpga_p,
  sync_from_fpga_n=> sync_from_fpga_n,

  adc0_clka_p     => adc0_clka_p,
  adc0_clka_n     => adc0_clka_n,
  adc0_da_p       => adc0_da_p,
  adc0_da_n       => adc0_da_n,
  adc0_ovra_p     => adc0_ovra_p,
  adc0_ovra_n     => adc0_ovra_n,
  adc0_db_p       => adc0_db_p,
  adc0_db_n       => adc0_db_n,
  adc0_ovrb_p     => adc0_ovrb_p,
  adc0_ovrb_n     => adc0_ovrb_n,

  adc1_clka_p     => adc1_clka_p,
  adc1_clka_n     => adc1_clka_n,
  adc1_da_p       => adc1_da_p,
  adc1_da_n       => adc1_da_n,
  adc1_ovra_p     => adc1_ovra_p,
  adc1_ovra_n     => adc1_ovra_n,

  dac_sync_p      => dac_sync_p,
  dac_sync_n      => dac_sync_n,

  dac0_dclk_p     => dac0_dclk_p,
  dac0_dclk_n     => dac0_dclk_n,
  dac0_data_p     => dac0_data_p,
  dac0_data_n     => dac0_data_n,

  dac1_dclk_p     => dac1_dclk_p,
  dac1_dclk_n     => dac1_dclk_n,
  dac1_data_p     => dac1_data_p,
  dac1_data_n     => dac1_data_n,

  pg_m2c          => pg_m2c,
  prsnt_m2c_l     => prsnt_m2c_l

);


-------------------------------------------------------------------------------------
--synchronous processes
-------------------------------------------------------------------------------------


-------------------------------------------------------------------------------------
--asynchronous processes
-------------------------------------------------------------------------------------


-------------------------------------------------------------------------------------
--asynchronous mapping
-------------------------------------------------------------------------------------


end architecture arch_sip_fmc110   ; -- of sip_fmc110
